
// **********************************************************************
// *                                                                    *
// * SMBUS on the Intel PIIX4 and the Reliance OSB4 chipsets            *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************


/*----------------------* PRAGMAS *----------------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*----------------------* DEFINES *----------------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*---------------------*  HEADERS *----------------------------*/
/*-------------------------------------------------------------*/
#include "i2cos.h"
#include "i2cio.h"
#include "i2cpiix4.h"
#include "pci.h"

/*--------------------* LOCAL DATA *---------------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*--------------------* EXTERNAL DATA *------------------------*/
/*-------------------------------------------------------------*/
extern int MAX_RETRIES;
extern unsigned int SMBusBaseAddr;
extern int IsOSB4Found;
extern int IsCSB5Found;

// CSB5 Reset usage.
// Globals to allow me to save these registers from the beginning of the
// SMBus usage. If we have to reset the CSB5, then we need to restore the
// original data that was in these registers before we started.
// (The config register should have the SMBus enable bit already turned on.)
extern int CSB5bus;
extern int CSB5device;
extern unsigned long CSB5config;
extern unsigned long CSB5address;
extern unsigned long CSB5control;

/*----------------* EXTERNAL PROTOTYPES *----------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*-------------------* LOCAL PROTOTYPES *----------------------*/
/*-------------------------------------------------------------*/
I2CSTATUS PIIX4WaitForSMBus(void);
I2CSTATUS PIIX4TestSMBusStatus(void);
void CheckKibblesAndBits(void);
void CSB5ResetSMBusHost(void);
I2CSTATUS PIIX4ReceiveByte(BYTE, BYTE FAR *);
I2CSTATUS PIIX4SendByte(BYTE, BYTE);
I2CSTATUS PIIX4ReadDataByte(BYTE, BYTE, BYTE FAR *);
I2CSTATUS PIIX4WriteDataByte(BYTE, BYTE, BYTE);
I2CSTATUS PIIX4ReadDataWord(BYTE, BYTE, short FAR *);
I2CSTATUS PIIX4WriteDataWord(BYTE, BYTE, short);
I2CSTATUS PIIX4ReadDataBlock(BYTE, BYTE, BYTE *, int *);


//*******************************************************************************
//------------------------------N E W -------------------------------------------
//I2CSTATUS PIIX4H8ReadWrite(BYTE, BYTE *, int *, int);
//------------------------------N E W -------------------------------------------
//*******************************************************************************


I2CSTATUS PIIX4WriteDataBlock(BYTE, BYTE, BYTE *, int);
I2CSTATUS PIIX4QuickCommand(BYTE, BYTE);

/*-----------------------* FUNCTIONS *-------------------------*/
/*-------------------------------------------------------------*/

///////////////////////////////////////////////////////////////////////////
// PIIX4 Wait for SMBus                                       	         //
//									                                     //
// Notes: Waits for PIIX4 SMBus host and slave status register busy bits //
//        to become 0 (free).                                            //
//									                                     //
// On Entry:   None.                                            	     //
//									                                     //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4WaitForSMBus(void)
{
	int iCount = 0;
	
	// Check for a OSB4 or CSB5 chipset for some reason, the slave status bit is always 1 in the OSB4.
	if (IsOSB4Found || IsCSB5Found)  
	{
		// Check for SMBus activity via CMS for AOL (BPR added 05/31/01 for AOL fix)
		CheckKibblesAndBits ();
		
        debug_out(DETAILS_LVL,"PIIX4WaitForSMBus. Wait for PIIX4_SMB_STA_HST_BUSY not busy.\n");
		while ((INP (SMBusBaseAddr + PIIX4_SMB_HST_STA) & PIIX4_SMB_STA_HST_BUSY))
		{
			// Check for SMBus activity via CMS for AOL (BPR added 05/31/01 for AOL fix)
			CheckKibblesAndBits ();
			
			// added:
			// CSB5 Only!
			// Try 3 times to reset the SMBUS Host on the system.
			if ((iCount == 20) || (iCount == 40) || (iCount == 60))
			{
				debug_out(DETAILS_LVL, "PIIX4WaitForSMBus. Resetting CSB5 host controller\n");				
				CSB5ResetSMBusHost();
			}

			// Delay 500 us increments for a maximum of 40 ms before giving up.
			if (iCount == 80)
			{
				debug_out(DETAILS_LVL, "PIIX4WaitForSMBus. ERR_TIMEOUT (0x%x)\n", INP (SMBusBaseAddr + PIIX4_SMB_HST_STA));
				return (ERR_TIMEOUT);
			}
			
			DELAY(5000);	 // 500 us
			iCount++;
		}
	}
	else   // not an OSB4 or a CSB5
	{ 
		while ((INP (SMBusBaseAddr + PIIX4_SMB_HST_STA) & PIIX4_SMB_STA_HST_BUSY) ||
				(INP (SMBusBaseAddr + PIIX4_SMB_SLV_STA) & PIIX4_SMB_STA_SLV_BUSY))
		{
			if (iCount == MAX_DELAY)
			{
				// Kill the current transaction
				OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, PIIX4_SMB_CTL_KILL);
				DELAY (DELAY_TIME);
				OUTP (SMBusBaseAddr + PIIX4_SMB_HST_STA, INP (SMBusBaseAddr + PIIX4_SMB_HST_STA));
				//                OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, (PIIX4_SMB_CTL_KILL | PIIX4_SMB_CTL_START));
				//                OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, 0);
				
				debug_out(DETAILS_LVL,"PIIX4WaitForSMBus.  PIIX4 Found.  Transaction killed.  Return ERR_TIMEOUT.\n");
				return (ERR_TIMEOUT);
			}
			
			DELAY (DELAY_TIME);
			iCount++;
		}
	}
	
	// Clear all status bits
	// Note:  This is generally unnecessary as the SMBus is ready for use at this point
	//        however, a 1 bit written to any bit position but bit 0 will clear that bit
	//        position.
    debug_out(DETAILS_LVL,"PIIX4WaitForSMBus.  End func - success.  Clear PIIX4_SMB_HST_STA.  Return ERR_NO_ERROR.\n");
	OUTP (SMBusBaseAddr + PIIX4_SMB_HST_STA, INP (SMBusBaseAddr + PIIX4_SMB_HST_STA));
	
	return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Test SMBus Status                                       	     //
//									                                     //
// Notes: Checks SMBus host status register for completion of the        //
//        requested transaction.                                         //
//									                                     //
// On Entry:   None.                                            	     //
//									                                     //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4TestSMBusStatus(void)
{
	int iCount = 0;
	BYTE Status;
	
	// Wait for either host busy or SMBus interrupt bits to become set
    debug_out(DETAILS_LVL,"PIIX4TestSMBusStatus.  Wait for host busy or SMBus interrupt bits set.\n");	
	while (!(INP(SMBusBaseAddr + PIIX4_SMB_HST_STA) & 0x03))
	{
		// Delay 500 us increments for a maximum of 20 ms before giving up.
		if (iCount == 40)
		{
			debug_out(DETAILS_LVL, "PIIX4TestSMBusStatus.  ERR_TIMEOUT_HOST (0x%x)\n", INP (SMBusBaseAddr + PIIX4_SMB_HST_STA));
			return (ERR_TIMEOUT_HOST);
		}
		
		DELAY(5000);	 // 500 us
		iCount++;
	}
	
	// Wait for host busy bit to become 'not busy' (indicating that the SMBus transaction has completed)
    debug_out(DETAILS_LVL,"PIIX4TestSMBusStatus.  Wait for SMBus complete.\n"); 
	iCount = 0;
	while ((Status = INP(SMBusBaseAddr + PIIX4_SMB_HST_STA)) & 0x01)
	{
		// Delay 500 us increments for a maximum of 20 ms before giving up.
		if (iCount == 40)
		{
			debug_out(DETAILS_LVL, "PIIX4TestSMBusStatus. ERR_TIMEOUT (0x%x)\n", Status);
			return (ERR_TIMEOUT);
		}
		
		DELAY(5000);	 // 500 us
		iCount++;
	}
	
	// Clear all status bits
    debug_out(DETAILS_LVL,"PIIX4TestSMBusStatus.  Clear PIIX4_SMB_HST_STA.\n");	   
	OUTP (SMBusBaseAddr + PIIX4_SMB_HST_STA, INP (SMBusBaseAddr + PIIX4_SMB_HST_STA));
	
	// Check for successful completion, if not return proper error code
	if ((Status & 0x1C) != 0x00)
	{
		if (Status & 0x10)
		{
			debug_out(DETAILS_LVL, "PIIX4TestSMBusStatus. ERR_FAILED (0x%x)\n", Status);
			return (ERR_FAILED);
		}
		
		if (Status & 0x08)
		{
			debug_out(DETAILS_LVL, "PIIX4TestSMBusStatus. ERR_BUS_ERROR (0x%x)\n", Status);
			return (ERR_BUS_ERROR);
		}
		
		if (Status & 0x04)
		{
			debug_out(DETAILS_LVL, "PIIX4TestSMBusStatus. ERR_DEV_ERROR (0x%x)\n", Status);
			return (ERR_DEV_ERROR);
		}
	}
	
    debug_out(DETAILS_LVL,"PIIX4TestSMBusStatus.  End func - success.  Return ERR_NO_ERROR.\n");
	return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// Check Kibbles and Bits                                      	         //
//									                                     //
// Notes: This routine checks the OSB4's for SMBus activity by reading   //
//        the 'Kibbles and Bits' index register which is an offset into  //
//        the Client Management and Security (CMS) register.             //
//									                                     //
// On Entry:   None.                                            	     //
//									                                     //
// On Return:  None.                                                     //
//									                                     //
void CheckKibblesAndBits(void)
{
	int iCount = 0;
	unsigned char ReadByte = 0;
	
	if (IsOSB4Found)
	{
		// Request the 'Kibbles and Bits' register
		OUTP (OSB4_CMS_Index, OSB4_KibblesOffset);
		
		while (iCount < 32)
		{
			ReadByte = INP (OSB4_CMS_Data);
			iCount++;
			while (((ReadByte & 0x03) != 0x03) && (iCount < 32))
			{
				ReadByte = INP (OSB4_CMS_Data);
				iCount++;
			}
		}
	}
}


///////////////////////////////////////////////////////////////////////////
// CSB5 Reset SMBus Host                                      	         //
//									                                     //
// Notes: This routine resets the CSB5's SMBus host controller.          //
//        This will allow retries on the bus which were not allowed on   //
//        the OSB4.                                        .             //
//									                                     //
// On Entry:   None.                                            	     //
//									                                     //
// On Return:  None.                                                     //
//									                                     //
void CSB5ResetSMBusHost(void)
{
	unsigned long pcireg = 0;
	int x;
	
	if (IsCSB5Found)
	{	
    	//printk("***** In CSB5ResetSMBusHost START *****\n");
	    for(x=0; x<100 ;x++)  DELAY(1);   // delay 1 sec
		
        pcireg = PCI_GetConfigRegister(CSB5bus, CSB5device, 0, (CSB5_PCICONTROL>>2));
        pcireg |= (unsigned long) CSB5_SMBHOSTRESET;
        PCI_SetConfigRegister(CSB5bus, CSB5device, 0, (CSB5_PCICONTROL>>2), pcireg);
        pcireg &= (~((unsigned long) CSB5_SMBHOSTRESET));
        PCI_SetConfigRegister(CSB5bus, CSB5device, 0, (CSB5_PCICONTROL>>2), pcireg);

        PCI_SetConfigRegister(CSB5bus, CSB5device, 0, (PIIX4_SM_REG_BASE_ADDR>>2), CSB5address);
        PCI_SetConfigRegister(CSB5bus, CSB5device, 0, (PIIX4_SMBHSTCFG>>2), CSB5config);

        //printk("***** In CSB5ResetSMBusHost END *****\n");
        for(x=0; x<100 ;x++)  DELAY(1);   // delay 1 sec
                                                                                               
	}
}



///////////////////////////////////////////////////////////////////////////
// PIIX4 SMBus Read                                            	         //
//									                                     //
// Notes: Common read routine for PIIX4 on the SMBus. Includes dispatch to //
//        Receive Byte and Read byte/word protocol handlers.             //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4_SMBusRead(BYTE Address, BYTE Command, void FAR *BufferPtr, int Length, int RetryCount)
{
 BYTE ReturnByte;
 short ReturnWord;
 I2CSTATUS Status;
 int iCountIndex;

 // Initialize the user output buffer
 for (iCountIndex = 0; iCountIndex < Length; iCountIndex++)
      ((char FAR*)BufferPtr)[iCountIndex] = 0;

 // Check length indicating a read word request 
 if (Length == 2)
    {
     // Do the read word protocol
     Status = PIIX4ReadDataWord (Address, Command, &ReturnWord);
	 debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  Read WORD. Address = %xh, Command = %xh, ReturnWord = %xh.\n", Address, Command, ReturnWord);

     // If an error occured setup for a retry
     if (Status != ERR_NO_ERROR)
        {
			debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  Readg WORD failed.  Retry.\n");
//          KdPrint( ("I2CNT.SYS-PIIX4_SMBusRead: PIIX4ReadDataWord error\n") );

		 if (RetryCount >= MAX_RETRIES)
		 {
//			KdPrint( ("I2CNT.SYS-PIIX4_SMBusRead: PIIX4ReadDataWord err-MAX_RETRIES\n") );
			 debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  All retries failed.\n");
             return (Status);
		 }
         else
            {
             DELAY (10000);

             // Retry by calling ourself
             return PIIX4_SMBusRead (Address, Command, BufferPtr, Length, RetryCount + 1);
            }
        }
     // Save the returned word in the user buffer
	 debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  Returned WORD = %xh..\n", ReturnWord);	
     ((short FAR*)BufferPtr)[0] = ReturnWord;
    }
 else                     
    {
     // Do the read byte protocol
     Status = PIIX4ReadDataByte(Address, Command, &ReturnByte);
	 debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  Read BYTE. Address = %xh, Command = %xh, ReturnByte = %xh.\n", Address, Command, ReturnByte);	

     // If an error occured setup for a retry
     if (Status != ERR_NO_ERROR)
        {
		 debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  Read BYTE failed.  Retry.\n");
         if (RetryCount >= MAX_RETRIES)
		 {
			 debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  All retries failed.\n");
             return (Status);
		 }
         else
           {
            DELAY (10000);

            // Retry by calling ourself
            return PIIX4_SMBusRead (Address, Command, BufferPtr, Length, RetryCount + 1);
           }
        }

     // Save the returned byte in the user buffer
     ((char FAR *)BufferPtr)[0] = ReturnByte;
	 debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  Returned BYTE = %xh.\n", ReturnByte);

     // Implement multi-byte read as a 'send command then read byte' followed
     // by multiple 'receive byte' operations.  This should work fine for 
     // EEPROMs and devices which implement 'auto increment' registers. 
     for (iCountIndex = 1; iCountIndex < Length; iCountIndex++)
         {
         // Do the receive byte protocol
          Status = PIIX4ReceiveByte(Address, &ReturnByte);
		  debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  Read multi byte.  Returned BYTE = %xh.\n", ReturnByte); 

          // If an error occured setup for a retry
          if (Status != ERR_NO_ERROR)
             {
			    debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  Read multi bytes failed.  Retry.\n");
//				KdPrint( ("I2CNT.SYS-PIIX4_SMBusRead: PIIX4ReadDataByte error\n") );
              if (RetryCount >= MAX_RETRIES)
			  {
				  debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  All retries failed.\n");
//				KdPrint( ("I2CNT.SYS-PIIX4_SMBusRead: PIIX4ReadDataByte error-MAX_RETRIES\n") );
                  return (Status);
			  }
              else
                 {
                  DELAY (10000);
               
                  // Retry by calling ourself
                  return PIIX4_SMBusRead(Address, Command, BufferPtr, Length, RetryCount + 1);
                 }
             }

          // Save the returned byte in the user buffer
		  debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  Returned BYTE = %xh.\n", ReturnByte); 
          ((char FAR *)BufferPtr)[iCountIndex] = ReturnByte;
         }
    }

// KdPrint( ("I2CNT.SYS-PIIX4_SMBusRead: no error\n") );
 debug_out(DETAILS_LVL,"PIIX4_SMBusRead.  End func - success.  Return ERR_NO_ERROR.\n");
 return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 SMBus Write                                            	     //
//									                                     //
// Notes: Common write routine for PIIX4 on the SMBus. Includes dispatch //
//        to Write byte/word protocol handlers.                          //
//									                                     //
// On Entry:   Address, command, pointer to write buffer, buffer length  //
//             and retry count.                                          //
 //								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4_SMBusWrite(BYTE Address, BYTE Command, void FAR *BufferPtr, int Length, int RetrtCount)
{
 I2CSTATUS Status;
 int iIndex, iCount;
   
 // Check length indicating a write word request 
 if (Length == 2)
    {
     // Do the write word protocol
     Status = PIIX4WriteDataWord (Address, Command, ((short FAR *)BufferPtr)[0]);
	 debug_out(DETAILS_LVL,"PIIX4_SMBusWrite.  Write WORD.  Address = %xh, Command = %xh, Data = %xh.\n", Address, Command, ((short FAR *)BufferPtr)[0]);

     // If an error occured setup for a retry
     if (Status != ERR_NO_ERROR)
        {
			debug_out(DETAILS_LVL,"PIIX4_SMBusWrite. Write WORD failed.  Retry.\n");
//          KdPrint( ("I2CNT.SYS-PIIX4_SMBusWrite: PIIX4WriteDataWord error\n") );
         if (RetrtCount >= MAX_RETRIES)
		 {
			debug_out(DETAILS_LVL,"PIIX4_SMBusWrite. All retries failed.\n"); 
//          KdPrint( ("I2CNT.SYS-PIIX4_SMBusWrite: PIIX4WriteDataWord error-MAX_RETRIES\n") );
             return (Status);
		 }
         else
            {
             DELAY (10000);

             // Retry by calling ourself
             return PIIX4_SMBusWrite (Address, Command, BufferPtr, Length, RetrtCount + 1);
            }
        }
    }
 else
    {
     for (iIndex = 0; iIndex < Length; iIndex++)
         {
          // Do the write byte protocol
          Status = PIIX4WriteDataByte (Address, Command, ((char FAR *)BufferPtr)[iIndex]);
		  debug_out(DETAILS_LVL,"PIIX4_SMBusWrite.  Write BYTE.  Address = %xh, Command = %xh, Data = %xh.\n", Address, Command, ((short FAR *)BufferPtr)[iIndex]);
   
          // If an error occured setup for a retry
          iCount = 0;
          while (Status != ERR_NO_ERROR)
                {
				 debug_out(DETAILS_LVL,"PIIX4_SMBusWrite. Write BYTE failed.  Retry.\n");
                 iCount++;
//				 KdPrint( ("I2CNT.SYS-PIIX4_SMBusWrite: PIIX4WriteDataByte error\n") );

                 if (iCount == MAX_RETRIES)
				 {
					 debug_out(DETAILS_LVL,"PIIX4_SMBusWrite. All retries failed.\n");
//					KdPrint( ("I2CNT.SYS-PIIX4_SMBusWrite: PIIX4WriteDataByte error-MAX_RETRIES\n") );
                    return (Status);
				 }
                 else
                    {
                     DELAY (10000);

                     // Do the write byte protocol
                     Status = PIIX4WriteDataByte (Address, Command, ((char FAR *)BufferPtr)[iIndex]);
                    }
                }
         
          // Increment the 'command' field.  This makes multi-byte writes 
          // with length > 2 work on EEPROMs and LMxx.
		  debug_out(DETAILS_LVL,"PIIX4_SMBusWrite. Increment offset. offset = %xh.\n", Command);	
          Command++;
         }
    }

// KdPrint( ("I2CNT.SYS-PIIX4_SMBusWrite: no error\n") );
 debug_out(DETAILS_LVL,"PIIX4_SMBusWrite. End func - success.  Return ERR_NO_ERROR.\n");	
 return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 SMBus Receive                                        	         //
//									                                     //
// Notes: Control routine for the Receive Byte protocol handler.         //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4_SMBusReceive(BYTE Address, void FAR *BufferPtr, int Length, int RetryCount)
{
 BYTE ReturnByte;
 I2CSTATUS Status;
 int iIndex, iCount;

 // Initialize the user output buffer
 for (iIndex = 0; iIndex < Length; iIndex++)
      ((char FAR*)BufferPtr)[iIndex] = 0;

 for (iIndex = 0; iIndex < Length; iIndex++)
     {
     // Do the receive byte protocol
	  debug_out(DETAILS_LVL,"PIIX4_SMBusReceive. Receive BYTE.  Address = %xh, Returned byte = %xh.\n", Address, ReturnByte);
      Status = PIIX4ReceiveByte (Address, &ReturnByte);

      // If an error occured setup for a retry
      iCount = 0;
      while (Status != ERR_NO_ERROR)
            {
			 debug_out(DETAILS_LVL,"PIIX4_SMBusReceive.  Receive BYTE failed.  Retry.\n");
             ++iCount;
//			 KdPrint( ("I2CNT.SYS-PIIX4_SMBusReceive: PIIX4ReceiveByte error\n") );

             if (iCount == MAX_RETRIES)
			 {
				 debug_out(DETAILS_LVL,"PIIX4_SMBusReceive.  All retries failed.\n");
//				 KdPrint( ("I2CNT.SYS-PIIX4_SMBusReceive: PIIX4ReceiveByte error\n") );
                 return (Status);
			 }
             else
               {
                DELAY (10000);

                // Do the receive byte protocol
                Status = PIIX4ReceiveByte (Address, &ReturnByte);
               }
            }

      // Save the returned byte in the user buffer
	  debug_out(DETAILS_LVL,"PIIX4_SMBusReceive.  Returned BYTE = %xh..\n", ReturnByte);	
      ((char FAR *)BufferPtr)[iIndex] = ReturnByte;
//	  KdPrint( ("I2CNT.SYS-PIIX4_SMBusReceive: received byte val \n"));

     }

// KdPrint( ("I2CNT.SYS-PIIX4_SMBusReceive: no error\n") );
 debug_out(DETAILS_LVL,"PIIX4_SMBusReceive.  End func - success.  Return ERR_NO_ERROR.\n");	 
 return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 SMBus Send                                           	         //
//									                                     //
// Notes: Control routine for the Send Byte protocol handler.            //
//									                                     //
// On Entry:   Address and command.                                      //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4_SMBusSend(BYTE Address, BYTE Command)
{
 I2CSTATUS Status;
 
 debug_out(DETAILS_LVL,"PIIX4_SMBusSend.  Address = %xh, Command = %xh.\n", Address, Command);
 Status = PIIX4SendByte(Address, Command);

 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 SMBus Read Block                                      	         //
//									                                     //
// Notes: Control routine for the Block Read protocol handler.           //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4_SMBusReadBlock(BYTE Address, BYTE Command, BYTE FAR *BufferPtr, int *Length, int RetryCount)
{
 I2CSTATUS Status;

	debug_out(DETAILS_LVL,"PIIX4_SMBusReadBlock.  Address = %xh, Command = %xh, BufferPtr = %xh, Length = %dd.\n", Address, Command, *BufferPtr, *Length);
	Status = PIIX4ReadDataBlock (Address, Command, BufferPtr, Length);

 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 H8 Read Write                                      	         //
//									                                     //
// Notes: Control routine for the H8 (firmware fix) read/write handler.  //
//									                                     //
// On Entry:   Address, command, pointer to read/write buffer, buffer    //
//             length and retry count.                                   //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4_H8ReadWrite(BYTE Address, BYTE FAR *BufferPtr, int *Length, int RetryCount)
{
 BYTE ReturnByte = 0;
 I2CSTATUS Status;
 int iIndex = 0;
 int iCount = 0;

 // Do the write of the data.  Very easy, just one byte at a time.
 for (iIndex = 0; iIndex < *Length; iIndex++)
     {
	  debug_out(DETAILS_LVL,"PIIX4_H8ReadWrite. Write one byte at a time.  Address = %xh, Data = %xh.\n", Address, ((BYTE*)BufferPtr)[iIndex]);
      Status = PIIX4_SMBusSend (Address, ((BYTE*)BufferPtr)[iIndex]);
      if (Status != ERR_NO_ERROR)
          return (Status);

      // 100 us delay for H8 firmware >= version 8
      DELAY (1000);
     }

 // Get the first return byte which is a length byte.
 Status = PIIX4ReceiveByte (Address, &ReturnByte);
 debug_out(DETAILS_LVL,"PIIX4_H8ReadWrite. First BYTE - length.  Address = %xh, Return byte = %xh.\n", Address, ReturnByte);
 // If an error occured setup for a retry
 iIndex = 0;
 iCount = 0;
 while (Status != ERR_NO_ERROR)
       {
		debug_out(DETAILS_LVL,"PIIX4_H8ReadWrite.  Receive BYTE.  Retry.\n");
        ++iCount;

        if (iCount == MAX_RETRIES)
  		   {
			 debug_out(DETAILS_LVL,"PIIX4_H8ReadWrite.  All retries failed.\n");
             return (Status);
		   }
        else
           {
            DELAY (10000);

            // Do the receive byte protocol
            Status = PIIX4ReceiveByte (Address, &ReturnByte);
           }
       }

 // Save the returned byte in the user buffer
 debug_out(DETAILS_LVL,"PIIX4_H8ReadWrite. Returned BYTE = %xh.\n", ReturnByte);
 *Length = ReturnByte;
 ((char FAR *)BufferPtr)[iIndex] = ReturnByte;
 iIndex++;

 // Now, get the data.  This is done up to the length count.
 // The return buffer can only handle 32 bytes and this includes the length byte, so max is 0x1f here.
	   
  if ((Status == ERR_NO_ERROR) && (*Length < 0xFF))    //d36489 and d38299
    {
     while (iIndex <= *Length)
		   {
	 	    Status = PIIX4ReceiveByte (Address, &ReturnByte);
            debug_out(DETAILS_LVL,"PIIX4_H8ReadWrite.  Received %xh (BYTE) from %xh.\n", ReturnByte, Address);
		    iCount = 0;
		    while (Status != ERR_NO_ERROR)
		     	  {
					debug_out(DETAILS_LVL,"PIIX4_H8ReadWrite.  Receive BYTE failed.  Retry. \n");
				    ++iCount;

				   if (iCount == MAX_RETRIES)
					  {
					   debug_out(DETAILS_LVL,"PIIX4_H8ReadWrite.  All retries failed\n");
				   	   return (Status);
					  }
				   else
				      {
				   	   DELAY (10000);

					   // Do the receive byte protocol
					   Status = PIIX4ReceiveByte (Address, &ReturnByte);
					  }
				  }
		   debug_out(DETAILS_LVL,"PIIX4_H8ReadWrite. Returned BYTE = %xh.\n", ReturnByte);
		   ((char FAR *)BufferPtr)[iIndex] = ReturnByte;
	  	   iIndex++;
		  }
    }
 
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 H8 Write                                            	         //
//									                                     //
// Notes: Control routine for the H8 (firmware fix) write handler.       //
//									                                     //
// On Entry:   Address, command, pointer to write buffer, buffer length  //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4_H8Write(BYTE Address, BYTE FAR *BufferPtr, int Length, int RetryCount)
{
 I2CSTATUS Status = ERR_NO_ERROR;
 int iIndex = 0;

 // Do the write of the data.  Very easy, just one byte at a time.
 for (iIndex = 0; iIndex < Length; iIndex++)
     {
      Status = PIIX4_SMBusSend (Address, ((BYTE*)BufferPtr)[iIndex]);
	  debug_out(DETAILS_LVL,"PIIX4_H8Write.  Address = %xh, Data = %xh.\n", Address, ((BYTE*)BufferPtr)[iIndex]);
      if (Status != ERR_NO_ERROR)
          return (Status);

      // 100 us delay for H8 firmware >= version 8
      DELAY (1000);
     }
 
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 SMBus Write Block                                     	         //
//									                                     //
// Notes: Control routine for the Block Write protocol handler.          //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4_SMBusWriteBlock(BYTE Address, BYTE Command, BYTE FAR *BufferPtr, int Length, int RetryCount)
{
 I2CSTATUS Status;

 Status = PIIX4WriteDataBlock (Address, Command, BufferPtr, Length);
 debug_out(DETAILS_LVL,"PIIX4_SMBusWriteBlock.  Address = %xh, Command = %xh, BufferPtr = %xh, Length = %dd.\n", Address, Command, *BufferPtr, Length);

 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 SMBus Quick Command                                  	         //
//									                                     //
// Notes: Control routine for the Quick Command protocol handler.        //
//									                                     //
// On Entry:   Address, command.                                         //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4_SMBusQuickCommand(BYTE Address, BYTE Command)
{
 I2CSTATUS Status;

 Status = PIIX4QuickCommand (Address, Command);
 debug_out(DETAILS_LVL,"PIIX4_SMBusQuickCommand.  Address = %xh, Command = %xh.\n", Address, Command);

 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Receive Byte                                          	         //
//									                                     //
// Notes: Reads data from the SMBus address in receive byte mode.        //
//									                                     //
// On Entry:   Address, pointer to read buffer.                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4ReceiveByte(BYTE Address, BYTE FAR* ReadByte)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = PIIX4WaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with read transfer
 debug_out(DETAILS_LVL,"PIIX4ReceiveByte.  Configure PIIX4_SMB_HST_ADD for read from %xh.\n", Address);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_ADD, (Address | 0x01));

 // Setup the host control register (byte receive mode)
 debug_out(DETAILS_LVL,"PIIX4ReceiveByte.  Configure PIIX4_SMB_HST_CTL - BYTE transaction and start transaction.\n");
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, (PIIX4_SMB_CTL_BYTE | PIIX4_SMB_CTL_START));

 // Check to the see the transaction completed successfully, if not return the error
 if ((Status = PIIX4TestSMBusStatus ()) != ERR_NO_ERROR)
     return (Status);

 // Get the read byte
 *ReadByte = INP (SMBusBaseAddr + PIIX4_SMB_HST_D0);
 debug_out(DETAILS_LVL,"PIIX4ReceiveByte.  Read BYTE successful.  PIIX4_SMB_HST_D0 = %xh.\n", *ReadByte);

 // Return the transaction status
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Send Byte                                             	         //
//									                                     //
// Notes: Writes data to the SMBus address in send byte mode.            //
//									                                     //
// On Entry:   Address and command.                               	     //
//									                                     //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4SendByte(BYTE Address, BYTE Command)
{
 I2CSTATUS Status;

 debug_out(DETAILS_LVL,"PIIX4SendByte.  Start.\n");
	
 // Check to see if we can use the SMBus, if not return the error
 if ((Status = PIIX4WaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);
 
 // Setup the slave device address
 debug_out(DETAILS_LVL,"PIIX4SendByte.  Configure PIIX4_SMB_HST_ADD with device address.  Address = %xh.\n", Address);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_ADD, (Address & 0xFE));

 // Setup the host command register
 debug_out(DETAILS_LVL,"PIIX4SendByte.  Configure PIIX4_SMB_HST_CMD offset.  offset = %xh.\n", Command);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CMD, Command);
      
 // Setup the host control register (byte send mode)
 debug_out(DETAILS_LVL,"PIIX4SendByte.  Configure PIIX4_SMB_HST_CTL - BYTE transaction and start transaction.\n");
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, (PIIX4_SMB_CTL_BYTE | PIIX4_SMB_CTL_START));

 // Return the transaction status
 return (PIIX4TestSMBusStatus ());
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Read Data Byte                                       	         //
//									                                     //
// Notes: Reads data from the SMBus address in byte mode.                //
//									                                     //
// On Entry:   Address, command, pointer to read buffer.                 //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4ReadDataByte(BYTE Address, BYTE Command, BYTE FAR *ReadByte)
{
 I2CSTATUS Status;


 //DBGOUT (("ReadDataByte: Address = 0x%02X, Command  = 0x%02X\n", Address, Command));


 // Check to see if we can use the SMBus, if not return the error
 if ((Status = PIIX4WaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with read transfer
 debug_out(DETAILS_LVL,"PIIX4ReadDataByte.  Configure PIIX4_SMB_HST_ADD for read from %xh.\n", Address);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_ADD, (Address | 0x01));

 // Setup the host command register
 debug_out(DETAILS_LVL,"PIIX4ReadDataByte.  Configure PIIX4_SMB_HST_CMD with offset.  offset = %xh.\n", Command);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CMD, Command);

 // Setup the host control register (byte mode)
 debug_out(DETAILS_LVL,"PIIX4ReadDataByte.  Configure PIIX4_SMB_HST_CTL - BYTE data transaction and start transaction.\n");
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, (PIIX4_SMB_CTL_BYTE_DATA | PIIX4_SMB_CTL_START));
 
 // Check to the see the transaction completed successfully, if not return the error
 if ((Status = PIIX4TestSMBusStatus ()) != ERR_NO_ERROR)
 {
//	KdPrint( ("I2CNT.SYS-PIIX4ReadDataByte: PIIX4TestSMBusStatus error\n") );
     return (Status);
 }

 // Get the read byte
 *ReadByte = INP (SMBusBaseAddr + PIIX4_SMB_HST_D0);
 debug_out(DETAILS_LVL,"PIIX4ReadDataByte.  Read byte data successful.  PIIX4_SMB_HST_D0 = %xh.\n", *ReadByte);

 // Return the transaction status
//KdPrint( ("I2CNT.SYS-PIIX4ReadDataByte: no error\n") );
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Write Data Byte                                       	         //
//									                                     //
// Notes: Writes data to the SMBus address in byte mode.                 //
//									                                     //
// On Entry:   Address, command, byte to write.                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4WriteDataByte(BYTE Address, BYTE Command, BYTE WriteByte)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = PIIX4WaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with write transfer
 debug_out(DETAILS_LVL,"PIIX4WriteDataByte.  Configure PIIX4_SMB_HST_ADD to write to %xh.\n", Address);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_ADD, (Address & 0xFE));

 // Setup the host command register
 debug_out(DETAILS_LVL,"PIIX4WriteDataByte.  Configure PIIX4_SMB_HST_CMD with offset.  offset = %xh.\n", Command);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CMD, Command);

 // Setup the data register 0 with the write byte
 debug_out(DETAILS_LVL,"PIIX4WriteDataByte.  Write %xh to PIIX4_SMB_HST_D0.\n", WriteByte);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_D0, WriteByte);

 // Setup the host control register (byte mode)
 debug_out(DETAILS_LVL,"PIIX4WriteDataByte.  Configure PIIX4_SMB_HST_CTL - BYTE data transaction and start transaction.\n");
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, (PIIX4_SMB_CTL_BYTE_DATA | PIIX4_SMB_CTL_START));

 // Return the transaction status
 return (PIIX4TestSMBusStatus ());
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Read Data Word                                       	         //
//									                                     //
// Notes: Reads data from the SMBus address in word mode.                //
//									                                     //
// On Entry:   Address, command, pointer to read buffer.                 //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4ReadDataWord(BYTE Address, BYTE Command, short FAR *ReadWord)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = PIIX4WaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with read transfer
 debug_out(DETAILS_LVL,"PIIX4ReadDataWord.  Configure PIIX4_SMB_HST_ADD to read from %xh.\n", Address);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_ADD, (Address | 0x01));

 // Setup the host command register
 debug_out(DETAILS_LVL,"PIIX4ReadDataWord.  Configure PIIX4_SMB_HST_CMD with offset.  offset = %xh.\n", Command);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CMD, Command);

 // Setup the host control register (word mode)
 debug_out(DETAILS_LVL,"PIIX4ReadDataWord.  Configure PIIX4_SMB_HST_CTL - WORD transaction and start transaction.\n");
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, (PIIX4_SMB_CTL_WORD | PIIX4_SMB_CTL_START));

 // Check to the see the transaction completed successfully, if not return the error
 if ((Status = PIIX4TestSMBusStatus ()) != ERR_NO_ERROR)
 {
//	KdPrint( ("I2CNT.SYS-PIIX4ReadDataWord: PIIX4TestSMBusStatus error\n") );
     return (Status);
 }

 // Get the lower 8 bits of the read word
 ((char FAR*)ReadWord)[0] = INP (SMBusBaseAddr + PIIX4_SMB_HST_D0);
 debug_out(DETAILS_LVL,"PIIX4ReadDataWord.  PIIX4_SMB_HST_D0 = %xh (lower 8bits).\n", ((char FAR*)ReadWord)[0]);
 // Get the upper 8 bits of the read word
 ((char FAR*)ReadWord)[1] = INP (SMBusBaseAddr + PIIX4_SMB_HST_D1);
 debug_out(DETAILS_LVL,"PIIX4ReadDataWord.  PIIX4_SMB_HST_D1 = %xh (upper 8bits).\n", ((char FAR*)ReadWord)[1]);
 // Return the transaction status
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Write Data Word                                       	         //
//									                                     //
// Notes: Writes data to the SMBus address in word mode.                 //
//									                                     //
// On Entry:   Address, command, word to write.                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4WriteDataWord(BYTE Address, BYTE Command, short WriteWord)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = PIIX4WaitForSMBus ()) != ERR_NO_ERROR)
 {
//	KdPrint( ("I2CNT.SYS-PIIX4WriteDataWord: PIIX4WaitForSMBus error\n") );
     return (Status);
 }

 // Setup the slave device address with write transfer
 debug_out(DETAILS_LVL,"PIIX4WriteDataWord.  Configure PIIX4_SMB_HST_ADD for write to %xh.\n", Address); 
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_ADD, (Address & 0xFE));

 // Setup the host command register
 debug_out(DETAILS_LVL,"PIIX4WriteDataWord.  Configure PIIX4_SMB_HST_CMD with offset.  offset = %xh.\n", Command);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CMD, Command);

 // Setup the data register 0 with the lower 8 bits of the write word
 debug_out(DETAILS_LVL,"PIIX4WriteDataWord.  Write %xh (lower 8bits) to PIIX4_SMB_HST_D0.\n", ((char FAR *)&WriteWord)[0]);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_D0, ((char FAR *)&WriteWord)[0]);

 // Setup the data register 1 with the upper 8 bits of the write word
 debug_out(DETAILS_LVL,"PIIX4WriteDataWord.  Write %xh (upper 8bits) to PIIX4_SMB_HST_D1.\n", ((char FAR *)&WriteWord)[1]);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_D1, ((char FAR *)&WriteWord)[1]);

 // Setup the host control register (word mode)
 debug_out(DETAILS_LVL,"PIIX4WriteDataWord.  Configure PIIX4_SMB_HST_CTL - WORD transaction and start transaction.\n");
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, (PIIX4_SMB_CTL_WORD | PIIX4_SMB_CTL_START));

 // Return the transaction status
 return (PIIX4TestSMBusStatus ());
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Read Data Block                                      	         //
//									                                     //
// Notes: Reads data from the SMBus address in block mode.               //
//									                                     //
// On Entry:   Address, command, pointer to read data buffer and the     //
//             buffer length.                                      	     //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4ReadDataBlock(BYTE Address, BYTE Command, BYTE *Data, int *Length)
{
 I2CSTATUS Status;
 int iIndex = 0;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = PIIX4WaitForSMBus ()) != ERR_NO_ERROR)
 {
//	KdPrint( ("I2CNT.SYS-PIIX4ReadDataBlock: PIIX4WaitForSMBus error\n") );
     return (Status);
 }

 // Setup the slave device address with read transfer
 debug_out(DETAILS_LVL,"PIIX4ReadDataBlock.  Configure PIIX4_SMB_HST_ADD for read from %xh.\n", Address);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_ADD, (Address | 0x01));

 // Setup the host command register
 debug_out(DETAILS_LVL,"PIIX4ReadDataBlock.  Configure PIIX4_SMB_HST_CMD with offset.  offset = %xh.\n", Command);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CMD, Command);

 // Setup the host control register (block mode)
 debug_out(DETAILS_LVL,"PIIX4ReadDataBlock.  Configure PIIX4_SMB_HST_CTL - block transaction and start transaction.\n");
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, (PIIX4_SMB_CTL_BLOCK | PIIX4_SMB_CTL_START));

 // Check to the see the transaction completed successfully, if not return the error
 if ((Status = PIIX4TestSMBusStatus ()) != ERR_NO_ERROR)
 {
//	KdPrint( ("I2CNT.SYS-PIIX4ReadDataBlock: PIIX4TestSMBusStatus error\n") );
     return (Status);
 }

 // Get the length of the data to be read
 *Length = INP (SMBusBaseAddr + PIIX4_SMB_HST_D0);
 debug_out(DETAILS_LVL,"PIIX4ReadDataBlock.  Length of data to be read = %dd.\n", *Length);

 // Reset the block data register (index pointer) by reading the host control register
 INP (SMBusBaseAddr + PIIX4_SMB_HST_CTL); 

 // Get the data from the block data register
 while (iIndex < *Length)
       {
		Data[iIndex] = INP (SMBusBaseAddr + PIIX4_SMB_BLK_DAT);
		debug_out(DETAILS_LVL,"PIIX4ReadDataBlock.  Data read = %xh.\n", Data[iIndex]);
		iIndex++;
	   }

 // Return the transaction status
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Write Data Block                                      	         //
//									                                     //
// Notes: Writes data to the SMBus address in block mode.                //
//									                                     //
// On Entry:   Address, command, pointer to write data buffer and buffer //
//             length.                                             	     //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4WriteDataBlock(BYTE Address, BYTE Command, BYTE *Data, int Length)
{
 I2CSTATUS Status;
 int iIndex = 0;

 // Check the buffer length and adjust if necessary to the proper length.
 // NOTE:  In proper practice this test should just return an error code to the application.
 if (Length > 32)
     Length = 32;

 if (Length < 0)
     Length = 0;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = PIIX4WaitForSMBus ()) != ERR_NO_ERROR)
 {
//	KdPrint( ("I2CNT.SYS-PIIX4WriteDataBlock: PIIX4WaitForSMBus error\n") );
     return (Status);
 }

 // Setup the slave device address with write transfer
 debug_out(DETAILS_LVL,"PIIX4WriteDataBlock.  Configure PIIX4_SMB_HST_ADD for write to %xh.\n", Address);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_ADD, (Address & 0xFE));

 // Setup the host command register
 debug_out(DETAILS_LVL,"PIIX4WriteDataBlock.  Configure PIIX4_SMB_HST_CMD with offset.  offset = %xh.\n", Command);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CMD, Command);

 // Setup the number of bytes to write
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_D0, Length); 

 // Reset the block data register (index pointer) by reading the host control register
 INP (SMBusBaseAddr + PIIX4_SMB_HST_CTL); 

 // Load the data into the block data register
 while (iIndex < Length)
       {
        OUTP (SMBusBaseAddr + PIIX4_SMB_BLK_DAT, Data[iIndex]);
		debug_out(DETAILS_LVL,"PIIX4WriteDataBlock.  Write %xh to PIIX4_SMB_BLK_DAT.\n", Data[iIndex]);
        iIndex++;
       }

 // Setup the host control register (block bode)
 debug_out(DETAILS_LVL,"PIIX4WriteDataBlock.  Configure PIIX4_SMB_HST_CTL - block transaction and start transaction.\n");
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, PIIX4_SMB_CTL_BLOCK | PIIX4_SMB_CTL_START);

 // Return the transaction status
 return (PIIX4TestSMBusStatus ());
}

///////////////////////////////////////////////////////////////////////////
// PIIX4 Quick Command                                           	     //
//									                                     //
// Notes: Writes data to the SMBus address in quick command mode.        //
//									                                     //
// On Entry:   Address, command.                                         //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS PIIX4QuickCommand(BYTE Address, BYTE Command)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = PIIX4WaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address (Note: address has the command embedded in it)
 debug_out(DETAILS_LVL,"PIIX4QuickCommand.  Configure PIIX4_SMB_HST_ADD for %xh address.\n", Address);
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_ADD, Address);

 // Setup the host control register (quick mode)
 debug_out(DETAILS_LVL,"PIIX4QuickCommand.  Configure PIIX4_SMB_HST_CTL - PIIX4_SMB_CTL_QUICK and start transaction.\n");
 OUTP (SMBusBaseAddr + PIIX4_SMB_HST_CTL, (PIIX4_SMB_CTL_QUICK | PIIX4_SMB_CTL_START)); 

 // Return the transaction status
 return (PIIX4TestSMBusStatus ());
}
