
// **********************************************************************
// *                                                                    *
// * SMBUS on the Intel PIIX4 and the Reliance OSB4 chipsets            *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

/*---------------------*  HEADERS *----------------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*------------------------* DEFINES *--------------------------*/
/*-------------------------------------------------------------*/
// PIIX4 PCI function 3 registers
#define PIIX4_SM_REG_BASE_ADDR  0x90
#define PIIX4_SMBHSTCFG         0xd2            // SMBus Configuration register
#define PIIX4_PCICMD            0x04

// CSB5 additions
#define CSB5_SMBHOSTRESET      0x10				// SMBus/I2C Bus
#define CSB5_PCICONTROL        0x40             // PCI Control Register

#define PIIX4_START             0x40
#define PIIX4_KILL              0x02

// OSB4 PCI Client Management and Security (CMS) registers 
#define OSB4_MISC_REGISTER      0x41
#define OSB4_CMS_Enable         0x20
#define OSB4_CMS_Index          0x0c50
#define OSB4_CMS_Data           0x0c51
#define OSB4_KibblesOffset      0x12

//  PIIX4 SMBus register offsets from base address
#define PIIX4_SMB_HST_STA       0x00            // Host Status register
#define PIIX4_SMB_SLV_STA       0x01            // Slave Status register
#define PIIX4_SMB_HST_CTL       0x02            // Host control register
#define PIIX4_SMB_HST_CMD       0x03            // Host Command register
#define PIIX4_SMB_HST_ADD       0x04            // Host Address register
#define PIIX4_SMB_HST_D0        0x05            // Host Data 0 register
#define PIIX4_SMB_HST_D1        0x06            // Host Data 1 register
#define PIIX4_SMB_BLK_DAT       0x07            // Host Block data register

// Host and Slave Status register bit definitions
#define PIIX4_SMB_STA_HST_BUSY  0x01        
#define PIIX4_SMB_STA_SLV_BUSY  0x01   
              
// Host Control register bit definitions
#define PIIX4_SMB_CTL_QUICK     0x00
#define PIIX4_SMB_CTL_KILL      0x02
#define PIIX4_SMB_CTL_BYTE      0x04
#define PIIX4_SMB_CTL_BYTE_DATA 0x08
#define PIIX4_SMB_CTL_WORD      0x0C
#define PIIX4_SMB_CTL_BLOCK     0x14 
#define PIIX4_SMB_CTL_LAST_BYTE 0x20
#define PIIX4_SMB_CTL_START     0x40

//  PIIX4 macros
#define PIIX4_CLEAR_HOST_STATUS() OUTP( (SMBusBaseAddr + PIIX4_SMB_HST_STA), 0x1E)
#define PIIX4_CLEAR_SLV_STATUS() OUTP( (SMBusBaseAddr + PIIX4_SMB_SLV_STA), 0x3C)

/*--------------------* GLOBAL DATA *--------------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*-------------------* GLOBAL PROTOTYPES *---------------------*/
/*-------------------------------------------------------------*/
I2CSTATUS PIIX4_SMBusRead(BYTE, BYTE, void FAR *,int, int);
I2CSTATUS PIIX4_SMBusWrite(BYTE, BYTE, void FAR *,int, int);
I2CSTATUS PIIX4_SMBusReceive(BYTE, void FAR *, int, int);

I2CSTATUS PIIX4_SMBusSend(BYTE, BYTE);

I2CSTATUS PIIX4_SMBusReadBlock(BYTE, BYTE, BYTE FAR *,int *, int);
I2CSTATUS PIIX4_H8ReadWrite(BYTE, BYTE FAR *,int *, int);
I2CSTATUS PIIX4_H8Write(BYTE, BYTE FAR *, int , int);
I2CSTATUS PIIX4_SMBusWriteBlock(BYTE, BYTE, BYTE FAR *,int, int);
I2CSTATUS PIIX4_SMBusQuickCommand(BYTE, BYTE);

/*----------------------* PRAGMAS *----------------------------*/
/*-------------------------------------------------------------*/

// None ---------


