
// **********************************************************************
// *                                                                    *
// * Common code file for the SMBUS on the SIS Chip.                    *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

#ifndef I2CSIS_H
#define I2CSIS_H

// SIS PCI function 3 registers
#define SIS_SM_REG_BASE_ADDR 0x68
//#define SISSMBHSTCFG 0xd2       //  SMBus Configuration register
//#define SISPCICMD 0x04

#define SIS_START 0x10
#define SISKILL 0x20

//  SIS SMBus register offsets from base address
#define SIS_SMB_STS 0x80       //  host status register
#define SIS_SMB_EN 0x81       //  enabled register
#define SIS_SMB_CNT 0x82       //  control register
#define SIS_SMBHOST_CNT 0x83       //  host control register
#define SIS_SMB_ADDR 0x84       //  host address register
#define SIS_SMB_CMD 0x85       //  host command register
#define SIS_SMB_PCOUNT  0x86       //  processed byte count
#define SIS_SMB_COUNT  0x87       //  block byte count
#define SIS_SMB_BYTE0 0x88      //  host data 0 register
#define SIS_SMB_BYTE1 0x89      //  host data 1 register
#define SIS_SMB_BYTE2 0x8A      //  host data 2 register
#define SIS_SMB_BYTE3 0x8B      //  host data 3 register
#define SIS_SMB_BYTE4 0x8C      //  host data 4 register
#define SIS_SMB_BYTE5 0x8D      //  host data 5 register
#define SIS_SMB_BYTE6 0x8E      //  host data 6 register
#define SIS_SMB_BYTE7 0x8F      //  host data 7 register
#define SIS_SMBDEV_ADDR 0x90    //  slave address received
#define SIS_SMB_DB0 0x91      //  data byte 0 received
#define SIS_SMB_DB1 0x92      //  data byte 1 received
#define SIS_SMB_SAA 0x93      //  slave alias address

// Host Status Register bit definitions
#define SIS_HOST_BUSY 0x1        
        
// Slave Status Register bit definitions
#define SIS_SLV_BUSY  0x2       

//  SIS macros
#define SIS_CLEAR_HOST_STATUS()  OUTP( (SMBusBaseAddr + SIS_SMB_STS), 0xfe);
		
//#define SISCLEAR_SLV_STATUS()  OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), 0x3c)


// Exported functions
I2CSTATUS I2CSISPointerSetThenRead2(BYTE slave_addr, BYTE command_code, void FAR *byte_read,
                                      int number_bytes_read, int number_retries);
I2CSTATUS I2CSISRead2(BYTE slave_addr, void FAR *byte_read, int number_bytes_read, int number_retries);
I2CSTATUS I2CSISPointerSetThenWrite2(BYTE slave_addr, BYTE command_code, void FAR *bytes_written,
                                       int number_bytes_written, int number_retries);
I2CSTATUS SISReadBlock(BYTE addr, BYTE pointer, BYTE FAR *retdata, short FAR *count);
I2CSTATUS SISWriteBlock(BYTE addr, BYTE pointer, BYTE FAR *data, short FAR count);
I2CSTATUS SISKillCommand(void);

#endif  // I2CSIS_H
