
// ************************************************************************
// *                                                                      *
// * IBMSMB dll with H8 support functions                                 *
// *                                                                      *
// * Portions written by the IBM Director Development Team                *
// * (C) Copyright IBM Corporation, 1998-2003                             *
// *                                                                      *
// *  This software may be used and distributed according to the terms    *
// *  of the Lesser GNU Public License, incorporated herein by reference. *
// *                                                                      *
// ************************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <linux/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include "ibmsmb.h"
#include "dump.h"

#define DEVS 4
#define TRUE 1
#define FALSE 0
#define BYTE  unsigned char
#define UCHAR unsigned char
#define BOOL  unsigned char

#define NUMRETRIES        5
#define COMMANDCHECKSUM   0x100
#define COMMANDLENGTH     0x02


int i2c_opened = FALSE;
int H8Exists = TRUE;
int h8Address = 0;

int ibmsmb[DEVS]; /* Device file descriptors */
int DevIdx = 0;

//-----------------------------------------------------------------------------------------------
void InitxHexxPairxs( void )  
{
  return;
} 


//-----------------------------------------------------------------------------------------------
void klog(UCHAR *bufptr, int length)
{
   Req_Buf reqbuf;  
   int iretval;
	
   DevIdx = 0;
	
   // request buf address and commmand fields not used.	
   reqbuf.length = length;
   reqbuf.data = bufptr;   
  
   iretval = ioctl(ibmsmb[DevIdx],IBMSMB_IOCTEST2, &reqbuf);
	
}  


//-----------------------------------------------------------------------------------------------
void DumpBuffer( BYTE *start, int size, BYTE *label )
/* Usage:
 *       Limit the size to dump to 25 bytes and the label to 8 characters
 */
{
  return;
} 
	


//-----------------------------------------------------------------------------------------------
int I2C_Open()
{
   int modes[DEVS] = {  O_RDONLY, O_WRONLY, O_RDWR };
   // char *mode_names[DEVS] = { "r", "w", "rw" };
   char devname[256];

   // Force to a single device for now.
   DevIdx = 0;

   sprintf(devname,"/dev/ibmsmb%d",0);
   ibmsmb[DevIdx] = open(devname,modes[DevIdx%3]);
	
   if (ibmsmb[DevIdx] > 0) 
   {
      return TRUE;
   }
   else
   {
      return FALSE;
   }
   
}

//-----------------------------------------------------------------------------------------------
int I2C_Close()
{
   DevIdx = 0;
   //close(ibmsmb[DevIdx]);
   return TRUE;
}



//-----------------------------------------------------------------------------------------------
// Returns 0 on success, error code on failure
int I2C_SendCommandThenRead0(BYTE slave_addr, 
                             BYTE command_code, 
                             void *retval, int length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.command = command_code; 
      reqbuf.length = length;
      reqbuf.data = retval;   
  
      iretval = ioctl(ibmsmb[DevIdx],I2C_SEND_COMMAND_THEN_READ, &reqbuf);
      // printf("[%x] smb SO sending I2C_SEND_COMMAND_THEN_READ IOCTL RC = %x\n",I2C_SEND_COMMAND_THEN_READ,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;	
}



//-----------------------------------------------------------------------------------------------
// Returns 0 on success, error code on failure
int I2C_SendCommandThenRead(BYTE slave_addr, 
                            BYTE command_code, 
                            void *value, int length)
{
   int i;
   int retval;

   // Fix for the RF part -- only allows single byte reads
   for (i=0; i<length; i++)
   {
      retval = I2C_SendCommandThenRead0(slave_addr, (BYTE)(command_code+i), 
                                        ((char*)value)+i, 1);
      if (retval != 0)
         return retval;
   }
   return retval;
}



//-----------------------------------------------------------------------------------------------
int I2C_SendCommandThenWrite(BYTE slave_addr, 
                             BYTE command_code, 
                             void *value, int length)
{
      Req_Buf reqbuf;  
      int iretval;

      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.command = command_code; 
      reqbuf.length = length;
      reqbuf.data = value;

      iretval = ioctl(ibmsmb[DevIdx], I2C_SEND_COMMAND_THEN_WRITE, &reqbuf);
      // printf("[%x] smb SO sending I2C_SEND_COMMAND_THEN_WRITE IOCTL. RC = %x\n",I2C_SEND_COMMAND_THEN_WRITE,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;	

}



//-----------------------------------------------------------------------------------------------
int I2C_Receive(BYTE slave_addr, 
                void *retval, int length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.length = length;
      reqbuf.data = retval;
	
      iretval = ioctl(ibmsmb[DevIdx], I2C_RECEIVE, &reqbuf);
      // printf("[%x] smb SO sending I2C_RECEIVE IOCTL. RC = %x\n",I2C_RECEIVE,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;	
	
}




//-----------------------------------------------------------------------------------------------
int I2C_Send(BYTE slave_addr, 
             void *value, int length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.length = length;
      reqbuf.data = value;
	
      iretval = ioctl(ibmsmb[DevIdx], I2C_SEND, &reqbuf);
      // printf("[%x] smb SO sending I2C_SEND IOCTL. RC = %x\n",I2C_SEND,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;	
}




//-----------------------------------------------------------------------------------------------
unsigned int I2C_Block_Read(BYTE slave_addr,
                            BYTE command_code,
                            BYTE *value,
                            int *length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      *length = 32;
      reqbuf.addr = slave_addr; 
      reqbuf.command = command_code;
      reqbuf.length = *length;
      reqbuf.data = value;	
	
      iretval = ioctl(ibmsmb[DevIdx], I2C_SEND_COMMAND_THEN_READ_BLOCK, &reqbuf);
      // printf("[%x] smb SO sending I2C_SEND_COMMAND_THEN_READ_BLOCK IOCTL. RC = %x\n", I2C_SEND_COMMAND_THEN_READ_BLOCK, iretval);
      *length = reqbuf.length;  
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;
	
}




//-----------------------------------------------------------------------------------------------
unsigned int I2C_Block_Write(BYTE slave_addr,
                      BYTE command_code,
                      BYTE *value,
                      int length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.command = command_code;
      reqbuf.length = length;
      reqbuf.data = value;
	
      iretval = ioctl(ibmsmb[DevIdx], I2C_SEND_COMMAND_THEN_WRITE_BLOCK, &reqbuf);
      // printf("[%x] smb SO sending I2C_SEND_COMMAND_THEN_WRITE_BLOCK IOCTL. RC = %x\n",I2C_SEND_COMMAND_THEN_WRITE_BLOCK,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;		
	
}



//-----------------------------------------------------------------------------------------------
unsigned int H8Write(BYTE slave_addr,
                     BYTE *value,
                     int length)
{
	
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.length = length;
      reqbuf.data = value;
	
      iretval = ioctl(ibmsmb[DevIdx], H8_Write, &reqbuf);
      // printf("[%x] smb SO sending H8_Write IOCTL. RC = %x\n", H8_Write, iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;	
	
}



//-----------------------------------------------------------------------------------------------
unsigned int H8ReadWrite(BYTE slave_addr,
                         BYTE *value,
                         int *length)
{
	
      Req_Buf reqbuf;  
      int iretval;
	
      // Remove dump
      // DumpBuffer( value, *length, (BYTE *)"passed..");
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.length = *length;
      reqbuf.data = value;
	
      iretval = ioctl(ibmsmb[DevIdx], H8_ReadWrite, &reqbuf);
      // printf("[%x] smb SO sending H8_ReadWrite IOCTL. RC = %x\n", H8_ReadWrite, iretval);
      *length = reqbuf.length;  

      // Remove dump
      // DumpBuffer( value, *length, (BYTE *)"return..");
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;		
	
}


//-----------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------


//-----------------------------------------------------------------------------------------------
int H8SendCommandThenRead(int address, UCHAR command, void *buffer, int *retLength, BOOL responseReqd)
{

   int checkSumVal = 0;
   int iCheckSum = 0,i,maxNumRetries = 10;
   int cRetLen = 0;
   UCHAR bufferStr[32];
   int retVal = FALSE;


   if (!H8Exists)
      return (FALSE);

   iCheckSum = (int)COMMANDCHECKSUM - (int)command;
   
   //the next for loop added  to add a certain number of retries
   //to make sure that if one try failed then try again for another (for saphire quartz)
   //********************************************************
   for (i=0;i<maxNumRetries;i++)
   {
      //********************************************************
	  bufferStr[0] = (UCHAR)COMMANDLENGTH;
      bufferStr[1] = command;
      bufferStr[2] = (UCHAR)iCheckSum;

	   //Have to set the number of bytes to write.
	   cRetLen = 3;

	   //Command parameter not needed.
      if (H8ReadWrite(address,bufferStr,&cRetLen) == 0)
      {

         //LOTS of minor changes to accomidate new return values.
         if ((cRetLen - 2) > 0)
         {
            //check the checksum to make sure all data came back correctly
			// Data comes back in the format of:
			// 0 = amount of data received
			// 1...  = data received
            for (i = 1; i == cRetLen; i++)
               checkSumVal = checkSumVal + (int)bufferStr[i];
            checkSumVal &= 0xff;
            if (checkSumVal == 0)
            {
			   //Copy significant data to buffer, set length and return valid.
               memcpy(buffer,(const void *)&bufferStr[2], (int)cRetLen-2 );
               *retLength = (int) cRetLen-2;
               retVal = TRUE;
            }
         }
         else
         {
		    //Copy significant data to buffer, set length and return valid.
            memcpy(buffer,(const void *)&bufferStr[2], (int) cRetLen-2 );
            *retLength = (int) cRetLen-2;
            retVal = TRUE;
         }
		   break;
      }
      // Sleep(1000);   
   }

   return (retVal);

}



//-----------------------------------------------------------------------------------------------
int Read_Write(UCHAR *buffer, int *retLength, int dataLen, BOOL retAllData)
{
   int checkSumVal = 0;
   int iCheckSum = 0,i,maxNumRetries = 10;
   int cRetLen = 0;
   UCHAR bufferStr[256];
   int	retVal = FALSE;


   if (!H8Exists)
      return (FALSE);

   if (dataLen<=0 || dataLen > (sizeof(bufferStr)-2))
      return FALSE;
   
   bufferStr[0] = (UCHAR)dataLen+1;
   memcpy(&bufferStr[1],(const void *)buffer,dataLen);
   
   iCheckSum = (int)COMMANDCHECKSUM;
   for (i=1;i<dataLen+1;i++)
      iCheckSum -= (int)bufferStr[i];

   bufferStr[dataLen+1] = (UCHAR)iCheckSum;

   //the next for loop added to add a certain number of retries
   //to make sure that if one try failed then try again for another (saphire quartz)
   //********************************************************
   for (i=0;i<maxNumRetries;i++)
   {
      //********************************************************
	  //bufferStr[0] = (UCHAR)COMMANDLENGTH;
      //bufferStr[1] = command;
      //bufferStr[2] = (UCHAR)iCheckSum;

	   //Have to set the number of bytes to write.
	   cRetLen = dataLen+2;
	   
	   //Command parameter not needed.
      if (H8ReadWrite(h8Address,bufferStr,&cRetLen) == 0)
      {
         if (cRetLen > *retLength)
         {
           // The return length was greater than caller expects (possible hardware glitch - retry)
           // Sleep(1000);
           continue;
         }
	       
         //LOTS of minor changes to accomidate new return values.
         if ((cRetLen - 2) > 0)
         {
            //check the checksum to make sure all data came back correctly
			// Data comes back in the format of:
			// 0 = amount of data received
			// 1...  = data received
            for (i = 1; i == cRetLen; i++)
               checkSumVal = checkSumVal + (int)bufferStr[i];
            checkSumVal &= 0xff;
            if (checkSumVal == 0)
            {
			      //Copy significant data to buffer, set length and return valid.
               if (retAllData)
               {
                  memcpy(buffer,(const void *)&bufferStr[0], (int)cRetLen );
                  *retLength = (int) cRetLen;
               }
               else
               {
                  memcpy(buffer,(const void *)&bufferStr[2], (int)cRetLen-2 );
                  *retLength = (int) cRetLen-2;
               }
               retVal = TRUE;
            }
         }
         else
         {

		    //Copy significant data to buffer, set length and return valid.
		    memcpy(buffer,(const void *)&bufferStr[2], (int) cRetLen-2 );
            *retLength = (int) cRetLen-2;
            retVal = TRUE;
         }
		   break;
      }
      // Sleep(1000);  
    }


   return (retVal);
}




//-----------------------------------------------------------------------------------------------
int H8RawRead_Write(UCHAR *buffer, int *retLength, int dataLen)
{
   return (Read_Write(buffer, retLength, dataLen, TRUE));
}


//-----------------------------------------------------------------------------------------------
int H8Read_Write(UCHAR *buffer, int *retLength, int dataLen)
{
   return (Read_Write(buffer, retLength, dataLen, FALSE));
}



//-----------------------------------------------------------------------------------------------
int H8_write(UCHAR *buffer, int dataLen)
{
   int iCheckSum = 0,i,rc;
   UCHAR bufferStr[128];
   BOOL retVal = FALSE;
   int maxRetries = 10;


   if (!H8Exists)
      return (FALSE);

   if (dataLen<=0 || dataLen > (sizeof(bufferStr)-2))
      return FALSE;

   if (dataLen<=0 || dataLen > (sizeof(bufferStr)-2))
      return FALSE;

   bufferStr[0] = (UCHAR)(dataLen+1);
   memcpy(&bufferStr[1],(const void *)buffer,dataLen);
   iCheckSum = (int)COMMANDCHECKSUM;
   for (i=1;i<dataLen+1;i++)
      iCheckSum -= (int)bufferStr[i];
   bufferStr[dataLen+1] = (UCHAR)iCheckSum;

 
   for (i=0;i<maxRetries;i++)
   {
      if ((rc = H8Write(h8Address,bufferStr,2+dataLen)) == 0)
      {
		 // printf("ibmsmblib:H8Write: The command succeeded\n");
         retVal = TRUE;
         break;
      }
	   else
      {
         retVal = FALSE;
         // printf("ibmsmblib:H8Write: The command failed\n");
         // Sleep(1000);  
      }
	}

	return (retVal);
}




//-----------------------------------------------------------------------------------------------
int H8Dll_Init()
{
   int totRetries=0;
   UCHAR buffer[64];
   int retLen = 0;
   UCHAR addrList[] = {0x60,0x6e};
   UCHAR i;

   // Set up buffer dumping to the syslog
   InitxHexxPairxs(); 

   
   if (h8Address)
   {
      // printf("ibmsmblib:H8Dll_Init the dll was already initialzed\n");
      return (TRUE);
   }

		
   while (!(i2c_opened = I2C_Open()) && (totRetries++ < NUMRETRIES))
	   	sleep(0);
   if (!i2c_opened)
   {
	 return FALSE;
   }

    
   // printf("sizeof addrList [%d], first item [%x]\n",sizeof(addrList),addrList[0]);
   for (i=0;i<sizeof(addrList);i++)
   {
        // printf("checking at address [%x]\n",addrList[i]);
        if (H8SendCommandThenRead(addrList[i], 0x51, (void *)buffer, &retLen, TRUE))
        {
           //check the return length - should be 9 bytes

           if (retLen == 9)
           {
              h8Address = (int)addrList[i];
              H8Exists = TRUE;
              break;
           }
        }
    }

    if (!h8Address) 
	{
         H8Exists = FALSE;
         return FALSE;
	}

    // printf("ibmsmblib:DllInit H8 address = %x\n", h8Address);
	
    return TRUE;
}


/*  
//-----------------------------------------------------------------------------------------------
int main(int argc, char * argv[])
{
	int i;
	char buffer[256];

	I2C_Open();

          //  do some testing using "buffer[256]"

	I2C_Close();
}
*/
