// **********************************************************************
// *                                                                    *
// * Code for retrieving information about PCI devices.                 *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

#ifndef PCI_H
#define PCI_H

// I/O ports for accessing PCI Configuration registers
#define PCI_CONFIG_ADDR 0x0CF8
#define PCI_CONFIG_DATA 0x0CFC  

// PCI Configuration values
#define ALI  0x10B9   // VID for Acer Lab. chipsets
#define M1535 0x7101  // DID for PMU

#define AMD    0x1022  // VID for AMD chipsets
#define A8111  0x7468  // DID for Low Pin Count (LPC) bridge (function 0)

#define INTEL 0x8086  // VID for Intel
#define PIIX4 0x7110  // DID for PIIX4 Function 0
#define I443MX 0x7198 // DID for i443MX Function 3
#define ICH   0x2410  // DID for ICH Function 0
#define ICH2  0x2440  // DID for ICH2 Function 0
#define ICH2M 0x244C  // DID for ICH2M Function 0
#define ICH3 0x248C  // DID for ICH3 Function 3
#define ICH4 0x24C0  // DID for ICH3 Function 3
#define ICH5 0x24D0 
#define ICHS 0x25A1
#define ICH6        0x2640     // DID for Low Pin Count (LPC) bridge (Function 0)
#define ICH6M       0x2641     // DID for Low Pin Count (LPC) bridge (Function 0)
#define ICH6W       0x2642     // DID for Low Pin Count (LPC) bridge (Function 0)
#define ICH7        0x27B8     // DID for Low Pin Count (LPC) bridge (Function 0)
#define ICH7M       0x27B9     // DID for Low Pin Count (LPC) bridge (Function 0)
#define ICH7DH      0x27B0     // DID for Low Pin Count (LPC) bridge (Function 0) 
#define ICH_ESB2M   0x2670     // DID for Low Pin Count (LPC) bridge (Function 0) 
#define RELIANCE 0x1166
#define OSB4     0x0200
#define CSB5     0x0201
#define CSB6U    0x0203   // DID for PCI to ISA bridge (function 0)  d37424

#define VIA   0x1106  // VID for Via
#define PIPC  0x0596  // DID for PICP Function 0
#define V686  0x0686  // DID for PICP Function 0
#define SIS     0x1039  // VID for SIS
#define SISSMB  0x0630  // DID for 630 Function 0
#define IBM     0x1014  // VID for IBM
#define WISEMAN 0x00DC  // DID for Wiseman 
#define CONDOR  0x010F  // DID for Eagle/Condor/Falcon 

#define ALISMB_ENABLE    0x01
#define PIIX4SMB_ENABLE  0x01
#define ICHSMB_ENABLE  0x01
#define VIASMB_ENABLE  0x01
#define SISSMB_ENABLE  0x01

// Misc defines
#define PCI_MAX_BUSES 7
#define PCI_MAX_DEVICES 32

// Documentation macros
#define IN
#define OUT

// Macros
#define PCI_ADDR_INIT(addr)               (*(addr)) = 0x80000000
#define PCI_ADDR_SET_BUS(addr, bus)       (*(addr)) |= (((long)bus & 0xffL) << 16)
#define PCI_ADDR_SET_DEVICE(addr, dev)    (*(addr)) |= (((long)dev & 0x1fL) << 11)
#define PCI_ADDR_SET_FCTN(addr, fctn)     (*(addr)) |= (((long)fctn & 0x7L) << 8)
#define PCI_ADDR_SET_TARGET(addr, offset) (*(addr)) |= (((long)offset & 0x3fL) << 2)


int PCIWalk(int IN vid, int IN did, int OUT FAR *bus, int OUT FAR *device);
unsigned long PCI_GetConfigRegister(int IN bus, int IN device,
                                    int IN fctn, int IN target);

void PCI_SetConfigRegister(int IN bus, int IN device,
                           int IN fctn, int IN target, unsigned long IN data);

#endif // PCI_H

