// **********************************************************************
// *                                                                    *
// * SMBUS using the ACPI SMBus 2.0 interface                           *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************


//#include "i2cnt.h"
#include "i2cio.h"
#include "smbacpi.h"
#include "i2cos.h"
//#include "pci.h"


// Globals for ACPI SMBus 2.0 interface
USHORT SMBusACPISCAddr;
USHORT SMBusACPIDataAddr;

extern int MAX_RETRIES;


//
// Local prototypes
//
void ACPISetRegister( UCHAR, UCHAR );
UCHAR ACPIGetRegister( UCHAR );
void ACPIWaitForEvent(void);
I2CSTATUS ACPIStatusToI2CStatus( UCHAR );
I2CSTATUS ACPIReceiveByte( UCHAR, PUCHAR );
I2CSTATUS ACPISendByte( UCHAR, UCHAR );
I2CSTATUS ACPIReadDataByte( UCHAR, UCHAR, PUCHAR );
I2CSTATUS ACPIWriteDataByte( UCHAR, UCHAR, UCHAR );
I2CSTATUS ACPIReadDataWord( UCHAR, UCHAR, PUSHORT );
I2CSTATUS ACPIWriteDataWord( UCHAR, UCHAR, USHORT );
I2CSTATUS ACPIReadDataBlock( UCHAR, UCHAR, PUCHAR, PUCHAR );
I2CSTATUS ACPIWriteDataBlock( UCHAR, UCHAR, PUCHAR, UCHAR );
I2CSTATUS ACPIQuickCommand( UCHAR, UCHAR );



void ACPISetRegister(UCHAR regist, UCHAR data)
{
	UCHAR retval; 

  debug_out(DETAILS_LVL, "ACPISetRegister: regist = 0x%02X, data = 0x%02X\n", regist, data);

  //Clear Querys
	while (INP(SMBusACPISCAddr) & ACPI_SC_STA_SCIE)
	{
//		debug_out(DETAILS_LVL, "Query! %x\n", INP(SMBusACPISCAddr));
		OUTP(SMBusACPISCAddr, ACPI_SC_QR_EC);
		while ((INP(SMBusACPISCAddr) & (ACPI_SC_STA_OBF)) != ACPI_SC_STA_OBF)
		{
//			debug_out(DETAILS_LVL, "!OBF %x\n", INP(SMBusACPISCAddr));
		};
		retval = INP(SMBusACPIDataAddr);
	};
	
  // Wait until all buffers are flushed.
  while (INP(SMBusACPISCAddr) & (ACPI_SC_STA_IBF | ACPI_SC_STA_OBF))
	{
//		debug_out(DETAILS_LVL, "IBF/OBF! %x\n", INP(SMBusACPISCAddr));
	};

  // Send a write command to the embedded controller.
  OUTP(SMBusACPISCAddr, ACPI_SC_WR_EC);

  // Wait for the embedded controller to process to write.
  while (INP(SMBusACPISCAddr) & (ACPI_SC_STA_IBF))
	{
//		debug_out(DETAILS_LVL, "IBF! %x\n", INP(SMBusACPISCAddr));
	};

  // Write the register to the data port.
  OUTP (SMBusACPIDataAddr, regist);

  // Wait for the embedded controller to process to write.
	while (INP(SMBusACPISCAddr) & (ACPI_SC_STA_IBF))
	{
//		debug_out(DETAILS_LVL, "IBF! %x\n", INP(SMBusACPISCAddr));
	};

  // Write the data to the data port.
  OUTP (SMBusACPIDataAddr, data);

  // Wait for the embedded controller to process the write.
  while (INP(SMBusACPISCAddr) & (ACPI_SC_STA_IBF))
	{
//		debug_out(DETAILS_LVL, "IBF! %x\n", INP(SMBusACPISCAddr));
	};
}
 


UCHAR ACPIGetRegister(UCHAR regist)
{
	UCHAR retval;
	UCHAR data;

 debug_out(DETAILS_LVL, "ACPIGetRegister: regist = 0x%02X\n", regist);

  //Clear Querys
	while (INP(SMBusACPISCAddr) & ACPI_SC_STA_SCIE)
	{
		debug_out(DETAILS_LVL, "Query! %x\n", INP(SMBusACPISCAddr));
		OUTP(SMBusACPISCAddr, ACPI_SC_QR_EC);
		while ((INP(SMBusACPISCAddr) & (ACPI_SC_STA_OBF)) != ACPI_SC_STA_OBF)
		{
//			debug_out(DETAILS_LVL, "!OBF %x\n", INP(SMBusACPISCAddr));
		};
		retval = INP(SMBusACPIDataAddr);
	};

  // Wait until all buffers are flushed.
	while (INP(SMBusACPISCAddr) & (ACPI_SC_STA_IBF | ACPI_SC_STA_OBF))
	{
		debug_out(DETAILS_LVL, "IBF/OBF! %x\n", INP(SMBusACPISCAddr));
		INP(SMBusACPISCAddr);
	};

	// Send a read command to the embedded controller.
  OUTP(SMBusACPISCAddr, ACPI_SC_RD_EC);

  // Wait for the embedded controller to process read.
  while (INP(SMBusACPISCAddr) & (ACPI_SC_STA_IBF))
	{
//		debug_out(DETAILS_LVL, "IBF! %x\n", INP(SMBusACPISCAddr));
	};

	// Write the register to the data port.
  OUTP(SMBusACPIDataAddr, regist);

  // Wait for the embedded controller to process write.
  while ((INP(SMBusACPISCAddr) & (ACPI_SC_STA_OBF)) != ACPI_SC_STA_OBF)
	{
//		debug_out(DETAILS_LVL, "!OBF %x\n", INP(SMBusACPISCAddr));
	}

  // Read the data byte from the data port.
  data = INP(SMBusACPIDataAddr);
  debug_out(DETAILS_LVL, "ACPIGetRegister: data = 0x%02X\n", data);

	return data;
}



void ACPIWaitForEvent()
{
  USHORT iCount = 0;

	//Wait for Query
	//Clear Querys
  debug_out(DETAILS_LVL, "ACPIWaitForEvents\n");
	while ((INP(SMBusACPISCAddr) & ACPI_SC_STA_SCIE) == 0x00)
	{
    if (iCount == MAX_DELAY)
    {
      // Kill the current transaction
      debug_out(DETAILS_LVL, "SMBus status = %02X\n", ACPIGetRegister(ACPI_HST_STA));
      debug_out(DETAILS_LVL, "ERR_TIMEOUT in ACPIWaitForEvent. Status = 0x%02x\n", INP(SMBusACPISCAddr));
      return;
    }

    iCount++;
	};
} 



// 0x07       ACPI_STA_UNKNOWN_ERR = ERR_FAILED,
// 0x10            ACPI_STA_NO_DEV = ERR_NO_ACK
// 0x11           ACPI_STA_DEV_ERR = ERR_DEV_ERROR
// 0x12    ACPI_STA_CMD_ACC_DENIED = ERR_NOT_IMPLEMENTED
// 0x13  ACPI_STA_UNKNOWN_HOST_ERR = ERR_BUS_ERROR
// 0x17    ACPI_STA_DEV_ACC_DENIED = ERR_DEV_ERROR
// 0x18           ACPI_STA_TIMEOUT = ERR_TIMEOUT
// 0x19      ACPI_STA_BAD_PROTOCOL = ERR_NOT_IMPLEMENTED
// 0x1a          ACPI_STA_DEV_BUSY = ERR_DEVICE_TIMEOUT
// 0x1f           ACPI_STA_PEC_ERR = ERR_PEC_ERROR
// 0x80         ACPI_STA_BYTE_DONE = ERR_NO_ERROR
I2CSTATUS ACPIStatusToI2CStatus(
    UCHAR Status
    )
{
  I2CSTATUS map[] = {
      0, 0, 0, 0, 0, 0, 0, ERR_FAILED, 
      0, 0, 0, 0, 0, 0, 0, 0, 
      ERR_NO_ACK, ERR_DEV_ERROR, ERR_NOT_IMPLEMENTED, ERR_BUS_ERROR, 0, 0, 0, ERR_DEV_ERROR, 
      ERR_TIMEOUT, ERR_NOT_IMPLEMENTED, ERR_DEVICE_TIMEOUT, 0, 0, 0, 0, ERR_PEC_ERROR };

  if (Status < 0x20)
    return map[Status];
  else if (Status >= ACPI_STA_BYTE_DONE)
    return ERR_NO_ERROR;
  else
    return ERR_FAILED;
}



///////////////////////////////////////////////////////////////////////////
// ACPI SMBus Read                                            	         //
//									                                     //
// Notes: Common read routine for ICH on the SMBus. Includes dispatch to //
//        Receive Byte and Read byte/word protocol handlers.             //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPI_SMBusRead(
    UCHAR Address, 
    UCHAR Command, 
    PVOID BufferPtr, 
    UCHAR Length, 
    int RetryCount
    )
{
  UCHAR ReturnByte;
  USHORT ReturnWord;
  I2CSTATUS Status;
  UCHAR iCountIndex;

  debug_out(DETAILS_LVL, "ACPI_SMBusRead: Address = 0x%02X, Command = 0x%02X, Length = %d, RetryCount = %d\n", Address, Command, Length, RetryCount);

  // Initialize the user output buffer
  for (iCountIndex = 0; iCountIndex < Length; iCountIndex++)
    ((PUCHAR)BufferPtr)[iCountIndex] = 0;

  // Check length indicating a read word request 
  if (Length == 2)
  {
    // Do the read word protocol
    Status = ACPIReadDataWord (Address, Command, &ReturnWord);

    // If an error occured setup for a retry
    if (Status != ERR_NO_ERROR)
    {
      if (RetryCount >= MAX_RETRIES)
      {
        debug_out(DETAILS_LVL, "ACPI_SMBusRead: maximum retries exceeded, returning 0x%02X\n", Status);
        return Status;
      }
      else
      {
        DELAY (10000);

        // Retry by calling ourself
        return ACPI_SMBusRead(Address, Command, BufferPtr, Length, RetryCount + 1);
      }
    }
    // Save the returned word in the user buffer
    ((PUSHORT)BufferPtr)[0] = ReturnWord;
  }
  else
  {
    // Do the read byte protocol
    Status = ACPIReadDataByte(Address, Command, &ReturnByte);

    // If an error occured setup for a retry
    if (Status != ERR_NO_ERROR)
    {
      if (RetryCount >= MAX_RETRIES)
      {
        debug_out(DETAILS_LVL, "ACPI_SMBusRead: maximum retries exceeded, returning 0x%02X\n", Status);
        return(Status);
      }
      else
      {
        DELAY (10000);

        // Retry by calling ourself
        return ACPI_SMBusRead (Address, Command, BufferPtr, Length, RetryCount + 1);
      }
    }

    // Save the returned byte in the user buffer
    ((PUCHAR)BufferPtr)[0] = ReturnByte;

    // Implement multi-byte read as a 'send command then read byte' followed
    // by multiple 'receive byte' operations.  This should work fine for 
    // EEPROMs and devices which implement 'auto increment' registers. 
    for (iCountIndex = 1; iCountIndex < Length; iCountIndex++)
    {
      // Do the receive byte protocol
      Status = ACPIReceiveByte(Address, &ReturnByte);

      // If an error occured setup for a retry
      if (Status != ERR_NO_ERROR)
      {
        if (RetryCount >= MAX_RETRIES)
        {
          debug_out(DETAILS_LVL, "ACPI_SMBusRead: maximum retries exceeded, returning 0x%02X\n", Status);
          return(Status);
        }
        else
        {
          DELAY (10000);

          // Retry by calling ourself
          return ACPI_SMBusRead(Address, Command, BufferPtr, Length, RetryCount + 1);
        }
      }

      // Save the returned byte in the user buffer
      ((PUCHAR)BufferPtr)[iCountIndex] = ReturnByte;
    }
  }

  debug_out(DETAILS_LVL, "ACPI_SMBusRead: returning ERR_NO_ERROR\n");
  return ERR_NO_ERROR;
}



///////////////////////////////////////////////////////////////////////////
// ACPI SMBus Write                                            	         //
//									                                     //
// Notes: Common write routine for ICH on the SMBus. Includes dispatch   //
//        to Write byte/word protocol handlers.                          //
//									                                     //
// On Entry:   Address, command, pointer to write buffer, buffer length  //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPI_SMBusWrite(
    UCHAR Address, 
    UCHAR Command, 
    PVOID BufferPtr, 
    UCHAR Length, 
    int RetryCount
    )
{
  I2CSTATUS Status;
  UCHAR iIndex, iCount;

  debug_out(DETAILS_LVL, "ACPI_SMBusWrite: Address = 0x%02X, Command = 0x%02X, Length = %d, RetryCount = %d\n", Address, Command, Length, RetryCount);
  // Check length indicating a write word request 
  if (Length == 2)
  {
    // Do the write word protocol
    Status = ACPIWriteDataWord(Address, Command, ((PUCHAR)BufferPtr)[0]);

    // If an error occured setup for a retry
    if (Status != ERR_NO_ERROR)
    {
      if (RetryCount >= MAX_RETRIES)
      {
        debug_out(DETAILS_LVL, "ACPI_SMBusWrite: maximum retries exceeded, returning 0x%02X\n", Status);
        return Status;
      }
      else
      {
        DELAY (10000);

        // Retry by calling ourself
        return ACPI_SMBusWrite(Address, Command, BufferPtr, Length, RetryCount + 1);
      }
    }
  }
  else
  {
    for (iIndex = 0; iIndex < Length; iIndex++)
    {
      // Do the write byte protocol
      Status = ACPIWriteDataByte (Address, Command, ((PUCHAR)BufferPtr)[iIndex]);

      // If an error occured setup for a retry
      iCount = 0;
      while (Status != ERR_NO_ERROR)
      {
        iCount++;

        if (iCount == MAX_RETRIES)
        {
          debug_out(DETAILS_LVL, "ACPI_SMBusWrite: maximum retries exceeded, returning 0x%02X\n", Status);
          return Status;
        }
        else
        {
          DELAY (10000);

          // Do the write byte protocol
          Status = ACPIWriteDataByte (Address, Command, ((PUCHAR)BufferPtr)[iIndex]);
        }
      }

      // Increment the 'command' field.  This makes multi-byte writes 
      // with length > 2 work on EEPROMs and LMxx.  
      Command++;
    }
  }

  debug_out(DETAILS_LVL, "ACPI_SMBusWrite: returning ERR_NO_ERROR\n");
  return ERR_NO_ERROR;
}



///////////////////////////////////////////////////////////////////////////
// ACPI SMBus Receive                                           	         //
//									                                     //
// Notes: Control routine for the Receive Byte protocol handler.         //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPI_SMBusReceive(
    UCHAR Address, 
    PVOID BufferPtr, 
    UCHAR Length, 
    int RetryCount
    )
{
  UCHAR ReturnByte;
  I2CSTATUS Status;
  UCHAR iIndex, iCount;

  debug_out(DETAILS_LVL, "ACPI_SMBusReceive: Address = 0x%02X, Length = %d, RetryCount = %d\n", Address, Length, RetryCount);
//	return ERR_NOT_IMPLEMENTED;

///*
  // Initialize the user output buffer
  for (iIndex = 0; iIndex < Length; iIndex++)
    ((PUCHAR)BufferPtr)[iIndex] = 0;

  for (iIndex = 0; iIndex < Length; iIndex++)
  {
    // Do the receive byte protocol
    Status = ACPIReceiveByte (Address, &ReturnByte);

    // If an error occured setup for a retry
    iCount = 0;
    while (Status != ERR_NO_ERROR)
    {
      iCount++;

      if (iCount == MAX_RETRIES)
      {
        debug_out(DETAILS_LVL, "ACPI_SMBusReceive: maximum retries exceeded, returning 0x%02X\n", Status);
        return Status;
      }
      else
      {
        DELAY (10000);

        // Do the receive byte protocol
        Status = ACPIReceiveByte (Address, &ReturnByte);
      }
    }

    // Save the returned byte in the user buffer
    ((PUCHAR)BufferPtr)[iIndex] = ReturnByte;
  }

  debug_out(DETAILS_LVL, "ACPI_SMBusWrite: returning ERR_NO_ERROR\n");
  return ERR_NO_ERROR;
//*/
}



///////////////////////////////////////////////////////////////////////////
// ACPI SMBus Send                                           	         //
//									                                     //
// Notes: Control routine for the Send Byte protocol handler.            //
//									                                     //
// On Entry:   Address and command.                                      //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPI_SMBusSend(
    UCHAR Address, 
    UCHAR Command
    )
{
  I2CSTATUS Status;

  debug_out(DETAILS_LVL, "ACPI_SMBusSend: Address = 0x%02X, Command = 0x%02X\n", Address, Command);
  Status = ACPISendByte (Address, Command);
	Status = ERR_NOT_IMPLEMENTED;

  debug_out(DETAILS_LVL, "ACPI_SMBusSend: returning %d\n", Status);
  return Status;
}



///////////////////////////////////////////////////////////////////////////
// ACPI SMBus Read Block                                      	         //
//									                                     //
// Notes: Control routine for the Block Read protocol handler.           //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPI_SMBusReadBlock(
    UCHAR Address, 
    UCHAR Command, 
    PUCHAR BufferPtr, 
    PUCHAR Length, 
    int RetryCount
    )
{
  I2CSTATUS Status;

  debug_out(DETAILS_LVL, "ACPI_SMBusReadBlock: Address = 0x%02X, Command = 0x%02X, Length = %d, RetryCount = %d\n", Address, Command, Length, RetryCount);
  Status = ACPIReadDataBlock (Address, Command, BufferPtr, Length);
  Status = ERR_NOT_IMPLEMENTED;

  debug_out(DETAILS_LVL, "ACPI_SMBusReadBlock: returning %d\n", Status);
  return Status;
}



///////////////////////////////////////////////////////////////////////////
// ACPI SMBus Write Block                                      	         //
//									                                     //
// Notes: Control routine for the Block Write protocol handler.          //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPI_SMBusWriteBlock(
    UCHAR Address, 
    UCHAR Command, 
    PUCHAR BufferPtr, 
    UCHAR Length, 
    int RetryCount
    )
{
  I2CSTATUS Status;

  debug_out(DETAILS_LVL, "ACPI_SMBusWriteBlock: Address = 0x%02X, Command = 0x%02X, Length = %d, RetryCount = %d\n", Address, Command, Length, RetryCount);
  Status = ACPIWriteDataBlock (Address, Command, BufferPtr, Length);

  debug_out(DETAILS_LVL, "ACPI_SMBusWriteBlock: returning %d\n", Status);
  return Status;
}



///////////////////////////////////////////////////////////////////////////
// ICH SMBus Quick Command                                    	         //
//									                                     //
// Notes: Control routine for the Quick Command protocol handler.        //
//									                                     //
// On Entry:   Address, command.                                         //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPI_SMBusQuickCommand(
    UCHAR Address, 
    UCHAR Command
    )
{
  I2CSTATUS Status;

  debug_out(DETAILS_LVL, "ACPI_SMBusQuickCommand: Address = 0x%02X, Command = 0x%02X\n", Address, Command);
  Status = ACPIQuickCommand(Address, Command);

  debug_out(DETAILS_LVL, "ACPI_SMBusQuickCommand: returning %d\n", Status);
  return Status;
}



///////////////////////////////////////////////////////////////////////////
// ICH Receive Byte                                          	         //
//									                                     //
// Notes: Reads data from the SMBus address in receive byte mode.        //
//									                                     //
// On Entry:   Address, pointer to read buffer.                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPIReceiveByte(
    UCHAR Address, 
    PUCHAR ReadByte
    )
{
  UCHAR Status;

  debug_out(DETAILS_LVL, "ACPIReceiveByte: Address = 0x%02X, \n", Address);
  
  // Setup the slave device address with read transfer
  ACPISetRegister(ACPI_HST_ADDR, Address);

  // Setup the host control register (read byte mode)
  ACPISetRegister(ACPI_HST_CTL, ACPI_CTL_RCV_BYTE);

  // Check to the see the transaction completed successfully, if not return the error
  ACPIWaitForEvent();
  if ((Status = ACPIGetRegister(ACPI_HST_STA)) != ACPI_STA_BYTE_DONE)
  {
    debug_out(DETAILS_LVL, "ACPIReceiveByte: returning error from status = %d\n", Status);
    return ACPIStatusToI2CStatus(Status);
  }

  // Get the read byte
  *ReadByte = ACPIGetRegister(ACPI_HST_D0);

  debug_out(DETAILS_LVL, "ACPIReceiveByte: ReadByte = 0x%02X, returning ERR_NO_ERROR\n", *ReadByte);
  return ERR_NO_ERROR;
}



///////////////////////////////////////////////////////////////////////////
// ICH Send Byte                                             	         //
//									                                     //
// Notes: Writes data to the SMBus address in send byte mode.            //
//									                                     //
// On Entry:   Address and command.                               	     //
//									                                     //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPISendByte(
    UCHAR Address, 
    UCHAR Command
    )
{
  UCHAR Status;

  debug_out(DETAILS_LVL, "ACPISendByte: Address = 0x%02X, Command  = 0x%02X\n", Address, Command);
  
  // Setup the slave device address with read transfer
  ACPISetRegister(ACPI_HST_ADDR, Address);

  // Setup the host command register
  ACPISetRegister(ACPI_HST_CMD, Command);

  // Setup the host control register (send byte mode)
  ACPISetRegister(ACPI_HST_CTL, ACPI_CTL_SND_BYTE);

  // Check to the see the transaction completed successfully, if not return the error
  ACPIWaitForEvent();
  if ((Status = ACPIGetRegister(ACPI_HST_STA)) != ACPI_STA_BYTE_DONE)
  {
    debug_out(DETAILS_LVL, "ACPISendByte: returning error from status = %d\n", Status);
    return ACPIStatusToI2CStatus(Status);
  }

  // Return the transaction status
  debug_out(DETAILS_LVL, "ACPISendByte: returning ERR_NO_ERROR\n");
  return ERR_NO_ERROR;
}



///////////////////////////////////////////////////////////////////////////
// ICH Read Data Byte                                       	         //
//									                                     //
// Notes: Reads data from the SMBus address in byte mode.                //
//									                                     //
// On Entry:   Address, command, pointer to read buffer.                 //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPIReadDataByte(
    UCHAR Address, 
    UCHAR Command, 
    PUCHAR ReadByte
    )
{
  UCHAR Status;

  debug_out(DETAILS_LVL, "ACPIReadDataByte: Address = 0x%02X, Command  = 0x%02X\n", Address, Command);
  
  // Setup the slave device address with read transfer.
  // The address should be shifted left one bit and the LSB should be zero.
debug_out(DETAILS_LVL, "Set address register to 0x%02x\n", Address);
  ACPISetRegister(ACPI_HST_ADDR, Address);
Status = ACPIGetRegister(ACPI_HST_ADDR);
debug_out(DETAILS_LVL, "Read address back = 0x%02X\n", Status);

  // Setup the host command register
debug_out(DETAILS_LVL, "Set command register to 0x%02x\n", Command);
  ACPISetRegister(ACPI_HST_CMD, Command);
Status = ACPIGetRegister(ACPI_HST_CMD);
debug_out(DETAILS_LVL, "Read command back = 0x%02X\n", Status);

  // Setup the host control register (read byte mode)
debug_out(DETAILS_LVL, "Set control register to 0x%02x (read data byte)\n", ACPI_CTL_RD_BYTE);
  ACPISetRegister(ACPI_HST_CTL, ACPI_CTL_RD_BYTE);

  // Check to the see the transaction completed successfully, if not return the error
  ACPIWaitForEvent();
  if ((Status = ACPIGetRegister(ACPI_HST_STA)) != ACPI_STA_BYTE_DONE)
  {
    debug_out(DETAILS_LVL, "ACPIReadDataByte: returning error from status = %d\n", Status);
    return ACPIStatusToI2CStatus(Status);
  }

  // Get the read byte
  *ReadByte = ACPIGetRegister(ACPI_HST_D0);

  debug_out(DETAILS_LVL, "ACPIReadDataByte: ReadByte = 0x%02X, returning ERR_NO_ERROR\n", *ReadByte);
  return ERR_NO_ERROR;
}



///////////////////////////////////////////////////////////////////////////
// ICH Write Data Byte                                       	         //
//									                                     //
// Notes: Writes data to the SMBus address in byte mode.                 //
//									                                     //
// On Entry:   Address, command, byte to write.                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPIWriteDataByte(
    UCHAR Address, 
    UCHAR Command, 
    UCHAR WriteByte
    )
{
  UCHAR Status;

  debug_out(DETAILS_LVL, "ACPIWriteDataByte: Address = 0x%02X, Command  = 0x%02X\n", Address, Command);
  
  // Setup the slave device address with read transfer
  ACPISetRegister(ACPI_HST_ADDR, Address);

  // Setup the host command register
  ACPISetRegister(ACPI_HST_CMD, Command);

  // Setup the data register 0 with the write byte
  ACPISetRegister(ACPI_HST_D0, WriteByte);

  // Setup the host control register (write byte mode)
  ACPISetRegister(ACPI_HST_CTL, ACPI_CTL_WR_BYTE);

  // Check to the see the transaction completed successfully, if not return the error
  ACPIWaitForEvent();
  if ((Status = ACPIGetRegister(ACPI_HST_STA)) != ACPI_STA_BYTE_DONE)
  {
    debug_out(DETAILS_LVL, "ACPIWriteDataByte: returning error from status = %d\n", Status);
    return ACPIStatusToI2CStatus(Status);
  }

  // Return the transaction status
  debug_out(DETAILS_LVL, "ACPIWriteDataByte: returning ERR_NO_ERROR\n");
  return ERR_NO_ERROR;
}



///////////////////////////////////////////////////////////////////////////
// ICH Read Data Word                                       	         //
//									                                     //
// Notes: Reads data from the SMBus address in word mode.                //
//									                                     //
// On Entry:   Address, command, pointer to read buffer.                 //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPIReadDataWord(
    UCHAR Address, 
    UCHAR Command, 
    PUSHORT ReadWord
    )
{
  UCHAR Status;

  debug_out(DETAILS_LVL, "ACPIReadDataWord: Address = 0x%02X, Command  = 0x%02X\n", Address, Command);
  
  // Setup the slave device address with read transfer
  ACPISetRegister(ACPI_HST_ADDR, Address);

  // Setup the host command register
  ACPISetRegister(ACPI_HST_CMD, Command);

  // Setup the host control register (read word mode)
  ACPISetRegister(ACPI_HST_CTL, ACPI_CTL_RD_WORD);

  // Check to the see the transaction completed successfully, if not return the error
  ACPIWaitForEvent();
  if ((Status = ACPIGetRegister(ACPI_HST_STA)) != ACPI_STA_BYTE_DONE)
  {
    debug_out(DETAILS_LVL, "ACPIReadDataWord: returning error from status = %d\n", Status);
    return ACPIStatusToI2CStatus(Status);
  }

  // Get the two bytes in the word
  ((PUCHAR)ReadWord)[0] = ACPIGetRegister(ACPI_HST_D0);
  ((PUCHAR)ReadWord)[1] = ACPIGetRegister(ACPI_HST_D1);

  debug_out(DETAILS_LVL, "ACPIReadDataWord: ReadWord = 0x%04X, returning ERR_NO_ERROR\n", *ReadWord);
  return ERR_NO_ERROR;
}



///////////////////////////////////////////////////////////////////////////
// ICH Write Data Word                                       	         //
//									                                     //
// Notes: Writes data to the SMBus address in word mode.                 //
//									                                     //
// On Entry:   Address, command, word to write.                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPIWriteDataWord(
    UCHAR Address, 
    UCHAR Command, 
    USHORT WriteWord
    )
{
  UCHAR Status;

  debug_out(DETAILS_LVL, "ACPIWriteDataWord: Address = 0x%02X, Command  = 0x%02X\n", Address, Command);
  
  // Setup the slave device address with read transfer
  ACPISetRegister(ACPI_HST_ADDR, Address);

  // Setup the host command register
  ACPISetRegister(ACPI_HST_CMD, Command);

  // Setup the data registers with the write word
  ACPISetRegister(ACPI_HST_D0, ((PUCHAR)&WriteWord)[0]);
  ACPISetRegister(ACPI_HST_D1, ((PUCHAR)&WriteWord)[1]);

  // Setup the host control register (write word mode)
  ACPISetRegister(ACPI_HST_CTL, ACPI_CTL_WR_WORD);

  // Check to the see the transaction completed successfully, if not return the error
  ACPIWaitForEvent();
  if ((Status = ACPIGetRegister(ACPI_HST_STA)) != ACPI_STA_BYTE_DONE)
  {
    debug_out(DETAILS_LVL, "ACPIWriteDataWord: returning error from status = %d\n", Status);
    return ACPIStatusToI2CStatus(Status);
  }

  // Return the transaction status
  debug_out(DETAILS_LVL, "ACPIWriteDataWord: returning ERR_NO_ERROR\n");
  return ERR_NO_ERROR;
}



///////////////////////////////////////////////////////////////////////////
// ICH Read Data Block                                      	         //
//									                                     //
// Notes: Reads data from the SMBus address in block mode.               //
//									                                     //
// On Entry:   Address, command, pointer to read data buffer and the     //
//             buffer length.                                      	     //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPIReadDataBlock(
    UCHAR Address, 
    UCHAR Command, 
    PUCHAR Data, 
    PUCHAR Length
    )
{
  UCHAR Status, len, i;

  debug_out(DETAILS_LVL, "ACPIReadDataBlock: Address = 0x%02X, Command  = 0x%02X, Length = %d\n", Address, Command, Length);
  
  // Setup the slave device address with read transfer
  ACPISetRegister(ACPI_HST_ADDR, Address);

  // Setup the host command register
  ACPISetRegister(ACPI_HST_CMD, Command);

  // Setup the host control register (read word mode)
  ACPISetRegister(ACPI_HST_CTL, ACPI_CTL_RD_BLK);

  // Check to the see the transaction completed successfully, if not return the error
  ACPIWaitForEvent();
  if ((Status = ACPIGetRegister(ACPI_HST_STA)) != ACPI_STA_BYTE_DONE)
  {
    debug_out(DETAILS_LVL, "ACPIReadDataBlock: returning error from status = %d\n", Status);
    return ACPIStatusToI2CStatus(Status);
  }

  // Get the length of the block. This is the lower 5 bits of the bulk count 
	//   register. Since the 5 bits cannot contain the maximum length of 32, 
	//   a value of 32 is represented with a zero.
  len = (ACPIGetRegister(ACPI_HST_BCNT) & 0x1F);
	if (len == 0)
		len = 32;
  else if (len > 32)
  {
    debug_out(DETAILS_LVL, "ACPIReadDataBlock: length > 32, returning ERR_FAILED\n");
    return ERR_FAILED;
  }
  
  for (i = 0; i < len; i++)
    Data[i] = ACPIGetRegister(ACPI_HST_D0 + i);

  debug_out(DETAILS_LVL, "ACPIReadDataBlock: returning ERR_NO_ERROR\n");
  return ERR_NO_ERROR;
}



///////////////////////////////////////////////////////////////////////////
// ICH Write Data Block                                      	         //
//									                                     //
// Notes: Writes data to the SMBus address in block mode.                //
//									                                     //
// On Entry:   Address, command, pointer to write data buffer and buffer //
//             length.                                             	     //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ACPIWriteDataBlock(
    UCHAR Address, 
    UCHAR Command, 
    PUCHAR Data, 
    UCHAR Length
    )
{
  UCHAR Status, i;

  debug_out(DETAILS_LVL, "ACPIWriteDataBlock: Address = 0x%02X, Command  = 0x%02X, Length = %d\n", Address, Command, Length);
  
  // Setup the data registers with the block data.
  if (Length > 32)
  {
    debug_out(DETAILS_LVL, "ACPIWriteDataBlock: length > 32, returning ERR_FAILED\n");
    return ERR_FAILED;
  }
	else if (Length == 0)
	{
		debug_out(DETAILS_LVL, "ACPIWriteDataBlock: length = 0, returning ERR_FAILED\n");
		return ERR_FAILED;
	}
  
  // Setup the slave device address with read transfer
  ACPISetRegister(ACPI_HST_ADDR, Address);

  // Setup the host command register
  ACPISetRegister(ACPI_HST_CMD, Command);

	// Write data bytes to the data registers
	for (i = 0; i < Length; i++)
    ACPISetRegister(ACPI_HST_D0 + i, Data[i]);

	// Set the data length. This is the lower 5 bits of the bulk count register.
	//   Since the 5 bits cannot contain the maximum length of 32, a value of
	//   32 is represented with a zero.
	if (Length == 32)
		ACPISetRegister(ACPI_HST_BCNT, 0);   // this means 32 bytes
	else
		ACPISetRegister(ACPI_HST_BCNT, (UCHAR) Length);

  // Setup the host control register (write block mode)
  ACPISetRegister(ACPI_HST_CTL, ACPI_CTL_WR_BLK);

  // Check to the see the transaction completed successfully, if not return the error
  ACPIWaitForEvent();
  if ((Status = ACPIGetRegister(ACPI_HST_STA)) != ACPI_STA_BYTE_DONE)
  {
    debug_out(DETAILS_LVL, "ACPIWriteDataBlock: returning error from status = %d\n", Status);
    return ACPIStatusToI2CStatus(Status);
  }

  // Return the transaction status
  debug_out(DETAILS_LVL, "ACPIWriteDataBlock: returning ERR_NO_ERROR\n");
  return ERR_NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////
// ICH Quick Command                                           	         //
//									                                     //
// Notes: Writes data to the SMBus address in quick command mode.        //
//									                                     //
// On Entry:   Address, command.                                         //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //

I2CSTATUS ACPIQuickCommand(
    UCHAR Address, 
    UCHAR Command
    )
{
  UCHAR Status;

  debug_out(DETAILS_LVL, "ACPIQuickCommand: Address = 0x%02X, Command  = 0x%02X\n", Address, Command);
  
  // Setup the slave device address with read transfer
  ACPISetRegister(ACPI_HST_ADDR, Address);

  // Setup the host control register (write quick mode)
  ACPISetRegister(ACPI_HST_CTL, ACPI_CTL_WR_QUICK);

  // Check to the see the transaction completed successfully, if not return the error
  ACPIWaitForEvent();
  if ((Status = ACPIGetRegister(ACPI_HST_STA)) != ACPI_STA_BYTE_DONE)
  {
    debug_out(DETAILS_LVL, "ACPIQuickCommand: returning error from status = %d\n", Status);
    return ACPIStatusToI2CStatus(Status);
  }

  // Return the transaction status
  debug_out(DETAILS_LVL, "ACPIQuickCommand: returning ERR_NO_ERROR\n");
  return ERR_NO_ERROR;
}

