// **********************************************************************
// *                                                                    *
// * SMBUS using the ACPI SMBus 2.0 interface                           *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************


#ifndef _SMBACPI_H
#define _SMBACPI_H

// Defines needed for module port
#define UCHAR BYTE
#define USHORT unsigned short
#define PUCHAR BYTE FAR *
#define PUSHORT USHORT FAR *
#define PVOID void *


//
// Defines
//
// ICH PCI function 3 registers
#define AMD_SM_REG_BASE_ADDR  0x10   // SMBus Base Address register
#define AMD_SMB_INT_LINE	    0x3C   // SMBus Interrupt Line register
#define AMD_SMBHSTCFG         0x40   // SMBus Host Configuration register
#define AMD_PCICMD   	        0x04

#define AMD_SMB_ENABLE  1
                  
// Offsets from AMD SMBus base register
#define AMD_BASE_DATA  0x00   // Data register
#define AMD_BASE_SC    0x04   // Status / Command Register
#define AMD_BASE_CFG   0x08   // Config register

// ACPI command values
#define ACPI_SC_RD_EC  0x80   // Read Embedded Controller
#define ACPI_SC_WR_EC  0x81   // Write Embedded Controller
#define ACPI_SC_QR_EC  0x84   // Query Embedded Controller

// ACPI status values
#define ACPI_SC_STA_SCIE  0x20   // SCI Event pending
#define ACPI_SC_STA_IBF   0x02		// SCI Input Buffer Full
#define ACPI_SC_STA_OBF   0x01		// SCI Output Buffer Full

// ACPI SMBus register offsets from SMBus base address
#define ACPI_HST_CTL   0x00   // Protocol Register
#define ACPI_HST_STA   0x01   // Slave Status register
#define ACPI_HST_ADDR  0x02   // Address register
#define ACPI_HST_CMD   0x03   // Command register
#define ACPI_HST_D0    0x04   // Data 0 register
#define ACPI_HST_D1    0x05   // Data 1 register
#define ACPI_HST_BCNT  0x24   // Block Count register
           
// Host Status register bit definitions
#define ACPI_STA_UNKNOWN_ERR       0x07
#define ACPI_STA_NO_DEV            0x10
#define ACPI_STA_DEV_ERR           0x11
#define ACPI_STA_CMD_ACC_DENIED    0x12
#define ACPI_STA_UNKNOWN_HOST_ERR  0x13
#define ACPI_STA_DEV_ACC_DENIED    0x17
#define ACPI_STA_TIMEOUT           0x18
#define ACPI_STA_BAD_PROTOCOL      0x19
#define ACPI_STA_DEV_BUSY          0x1A
#define ACPI_STA_PEC_ERR           0x1F
#define ACPI_STA_BYTE_DONE         0x80   

// Host Control register bit definitions
#define ACPI_CTL_WR_QUICK  0x02
#define ACPI_CTL_RD_QUICK  0x03
#define ACPI_CTL_SND_BYTE  0x04
#define ACPI_CTL_RCV_BYTE  0x05
#define ACPI_CTL_WR_BYTE   0x06
#define ACPI_CTL_RD_BYTE   0x07
#define ACPI_CTL_WR_WORD   0x08
#define ACPI_CTL_RD_WORD   0x09
#define ACPI_CTL_WR_BLK    0x0A
#define ACPI_CTL_RD_BLK    0x0B
                       

//
// Global prototypes
//
I2CSTATUS ACPI_SMBusRead( UCHAR, UCHAR, PVOID, UCHAR, int );
I2CSTATUS ACPI_SMBusWrite( UCHAR, UCHAR, PVOID, UCHAR, int );
I2CSTATUS ACPI_SMBusReceive( UCHAR, PVOID, UCHAR, int );
I2CSTATUS ACPI_SMBusSend( UCHAR, UCHAR );
I2CSTATUS ACPI_SMBusReadBlock( UCHAR, UCHAR, PUCHAR, PUCHAR, int );
I2CSTATUS ACPI_SMBusWriteBlock( UCHAR, UCHAR, PUCHAR, UCHAR, int );
I2CSTATUS ACPI_SMBusQuickCommand( UCHAR, UCHAR );


#endif  // _SMBACPI_H

