
// **********************************************************************
// *                                                                    *
// * smbmain.c -- main module for IBM Linux SMBus driver                *
// *                                                                    *
// * Portions written by the IBM Director Development Team              *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

/* Other Portions:
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 */ 
 

#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif

#include <linux/config.h>
#include <linux/module.h>

/* modversion stuff: no #ifdef needed if 2.0 support is not needed */
// #ifdef CONFIG_MODVERSIONS
// #include <linux/modversions.h>
// #endif

#include <linux/kernel.h> /* printk() */
//#include <linux/malloc.h> /* kmalloc() */
#include <linux/fs.h>     /* everything... */
#include <linux/errno.h>  /* error codes */
#include <linux/types.h>  /* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h>        /* O_ACCMODE */

#include <asm/system.h>   /* cli(), *_flags */

#ifdef __x86_64__
#include <asm/ioctl32.h>
#endif

#include "i2cos.h"         /* local definitions */
#include "i2cio.h"         /* local definitions */
#include "ibmsmb.h"        /* local definitions */
#include "dump.h"          

// SLES 10 introduced unlocked_ioctl & compat_ioctl ....
// If HAVE_UNLOCKED_IOCTL is defined use unlocked_ioctl instead of ioctl
// If HAVE_COMPAT_IOCTL is defined use compat_ioctl instead of register/unregister _ioctl32_conversion functions

#ifdef __x86_64__
/*
 * The IOCTL cmd received from 32 bit compiled applications
 */

/*
extern int register_ioctl32_conversion( unsigned int cmd,
                int(*handler)(unsigned int, unsigned int, unsigned long,
			      struct file* ));
extern int unregister_ioctl32_conversion( unsigned int cmd );
*/
#endif



/* Kmem caches were not available in 2.0. Disallow compilation in that case */
#ifdef LINUX_20
#  error "Kmem_cache functions are not available in Linux-2.0"
#else
#  if defined(LINUX_22) && LINUX_VERSION_CODE < KERNEL_VERSION(2,2,18)
#    error "This module needs at least 2.2.18 to run"
#  endif
/* leave the #ifdef open, so 2.0 won't get other errors */


/*
 * No static symbols here, because register_symtab is called
 */

int ibmsmb_major =   IBMSMB_MAJOR;
int ibmsmb_devs =    IBMSMB_DEVS;    /* number of bare ibmsmb devices */
int ibmsmb_qset =    IBMSMB_QSET;
int ibmsmb_quantum = IBMSMB_QUANTUM;

/* These are deprecated. use new definition
MODULE_PARM(ibmsmb_major, "i");
MODULE_PARM(ibmsmb_devs, "i");
MODULE_PARM(ibmsmb_qset, "i");
MODULE_PARM(ibmsmb_quantum, "i");
*/
module_param(ibmsmb_major, int, 0444);
module_param(ibmsmb_devs, int, 0444);
module_param(ibmsmb_qset, int, 0444);
module_param(ibmsmb_quantum, int, 0444);

MODULE_AUTHOR("IBM");
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,9)
	MODULE_LICENSE("GPL");
#endif
// The 2 key buffers that are used when talking with user space - MADE LOCAL TO IOCTL FUNCT
// Req_Buf reqbuf;
// unsigned char kernel_databuf[KERNEL_DATABUF_SIZE];

IbmSmb_Dev *ibmsmb_devices; /* allocated in ibmsmb_init */

int ibmsmb_trim(IbmSmb_Dev *dev);

/* declare one cache pointer: use it for all devices */
kmem_cache_t *ibmsmb_cache;

unsigned int Debug_Level = 0;

char dumpLine[DUMP_LINE_SIZE];   



//-----------------------------------------------------------------------------------------------
void InitxHexxPairxs( void )  
{
  return;
} 



//-----------------------------------------------------------------------------------------------
void DumpBuffer( BYTE *start, int size, BYTE *label )
/* Usage:
 *       Limit the size to dump to 25 bytes and the label to 8 characters
 */
{
  return;
} 
	


//---------------------------------------------------------------------------------------
void debug_out (unsigned int MsgLevel,  char *fmt, ...) 
{
    va_list  argptr;         /* pointer to additional args */ 
	char     buffer[256];    /* IMPORTANT: Keep message size within reason so don't overflow this local buffer */
	
    va_start(argptr, fmt);
    // printk("In debug_out!!!\n");
	
	if (Debug_Level >= MsgLevel) {
		printk("IBMSMB-PID=%x...", current->pid);
        vsprintf(buffer, fmt, argptr);
		printk(buffer);
	}	
    va_end(argptr);
}



/*
 * Open and close
 */
//---------------------------------------------------------------------------------------
int ibmsmb_open (struct inode *inode, struct file *filp)
{
    int num = MINOR(inode->i_rdev);
    IbmSmb_Dev *dev; /* device information */

    debug_out(INTERFACE_LVL,"ibmsmb_open.  Enter.\n");
	
	/* accept only root */
    if (! capable(CAP_SYS_ADMIN))
	{
	    debug_out(INTERFACE_LVL,"ibmsmb_open.  Non-Root access attempt.\n");
		return -EACCES; 
	}

    /*  check the device number */
    if (num >= ibmsmb_devs) return -ENODEV;
    dev = &ibmsmb_devices[num];

    /* now trim to 0 the length of the device if open was write-only */
     if ( (filp->f_flags & O_ACCMODE) == O_WRONLY) {
        if (down_interruptible (&dev->sem))
            return -ERESTARTSYS;
        ibmsmb_trim(dev); /* ignore errors */
        up (&dev->sem);
    }

    /* and use filp->private_data to point to the device data */
    filp->private_data = dev;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
    MOD_INC_USE_COUNT;
#endif
    return 0;          /* success */
}


//---------------------------------------------------------------------------------------
int ibmsmb_release (struct inode *inode, struct file *filp)
{
    debug_out(INTERFACE_LVL,"ibmsmb_release.  Enter.\n");
	
	/* accept only root */
    if (! capable(CAP_SYS_ADMIN))
	{
	    debug_out(INTERFACE_LVL,"ibmsmb_release.  Non-Root access attempt.\n");
		return -EACCES; 
	}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
    MOD_DEC_USE_COUNT;
#endif
    return 0;
}



/*
 * Native ioctl() implementation
 */
//---------------------------------------------------------------------------------------
#if defined(HAVE_UNLOCKED_IOCTL)
long
ibmsmb_ioctl_unlocked(struct file *filp,
		      unsigned int cmd,
		      unsigned long arg)
#else
int
ibmsmb_ioctl(struct inode *inode,
	     struct file *filp,
	     unsigned int cmd,
	     unsigned long arg)
#endif
{
    IbmSmb_Dev *dev = filp->private_data;
    int err= 0, ret = 0;

    Req_Buf reqbuf;
    unsigned char kernel_databuf[KERNEL_DATABUF_SIZE];
	
    unsigned char addr = 0;
    unsigned char command = 0;
    unsigned long len = 0;         /* int ??? FxFx  */
    void *data = 0;
    I2CSTATUS status = 0;	     
	    
    // Lock the driver via a Mutex to force serialize access by different processes
    if (down_interruptible (&dev->sem)) {
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  Signal received while blocking.\n");
		    return -ERESTARTSYS;	
	}
    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  Enter.\n");
	
	/* accept only root */
    if (! capable(CAP_SYS_ADMIN))  {ret = -EACCES; goto Exit_ioctl;}	

    /* don't even decode wrong cmds: better returning  ENOTTY than EFAULT */
    if (_IOC_TYPE(cmd) != IBMSMB_IOC_MAGIC) {ret = -ENOTTY; goto Exit_ioctl;}
	    
    if (_IOC_NR(cmd) > IBMSMB_IOC_MAXNR) {ret = -ENOTTY; goto Exit_ioctl;}

    /*
     * the type is a bitmask, and VERIFY_WRITE catches R/W
     * transfers. Note that the type is user-oriented, while
     * verify_area is kernel-oriented, so the concept of "read" and
     * "write" is reversed
     */
    if (_IOC_DIR(cmd) & _IOC_READ)
        err = !access_ok(VERIFY_WRITE, (void *)arg, _IOC_SIZE(cmd));
    else if (_IOC_DIR(cmd) & _IOC_WRITE)
        err =  !access_ok(VERIFY_READ, (void *)arg, _IOC_SIZE(cmd));
    if (err) {ret = -EFAULT; goto Exit_ioctl;}
	    
    // FxFx: Need to add paramater validation
    
    memset(&reqbuf, 0x00, sizeof(Req_Buf));
    memset(&kernel_databuf[0], 0x00, KERNEL_DATABUF_SIZE);
    
    switch(cmd) {
		
      /*  USING THIS AS THE WAY TO SET DEBUG TRACE LEVEL */	  
      case IBMSMB_IOCTEST1:
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
	    // Reuse the addr field of the reqbuf as the way to specify the debug level
        // printk("Debug_Level before = %d\n", Debug_Level);
     	Debug_Level = reqbuf.addr; 
        // printk("Debug_Level after = %d\n", Debug_Level);
   
        break;
      
	  
      /*  USING THIS AS THE KLOG TEST POINT (for user space synchronized logging) */	  
      case IBMSMB_IOCTEST2:            
        // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    len = reqbuf.length;
	
        // Get the user data to syslog (buffered I/O)
        if (copy_from_user (kernel_databuf, reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
	    data = &kernel_databuf[0]; 

 	    // Write the data to the syslog if debug messages are enabled
    	if (Debug_Level >= INTERFACE_LVL) 
	        printk("%s\n", (unsigned char *)data);

        break;

	
      case I2C_RECEIVE:
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_RECEIVE IOCTL.\n");
      
        // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    len = reqbuf.length;
	    data = &kernel_databuf[0]; 
	
	    command = reqbuf.command;   /* FxFx: Not Used but keep for debug signature */
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);
	
 	    // Receive the data from the SmBus
        status = I2CReceive(addr, data, len);
        DumpBuffer((BYTE *)data, len, (BYTE *)"return..");
	
        // Return the data to the user (buffered I/O)
        if (copy_to_user (reqbuf.data, data, len)) {ret = -EFAULT; goto Exit_ioctl;}  
        break;
  
		
      case I2C_SEND_COMMAND_THEN_READ:      
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND_COMMAND_THEN_READ IOCTL.\n");
      
        // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    command = reqbuf.command; 
	    len = reqbuf.length;
	    data = &kernel_databuf[0]; 
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);
	
 	    // Read the data from the SmBus
        status = I2CReadData(addr, command, data, len);
        DumpBuffer((BYTE *)data, len, (BYTE *)"return..");
	
        // Return the data to the user (buffered I/O)
        if (copy_to_user (reqbuf.data, data, len)) {ret = -EFAULT; goto Exit_ioctl;}  
        break;
      
		
      case I2C_SEND_COMMAND_THEN_WRITE:      
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND_COMMAND_THEN_WRITE IOCTL.\n");
      
        // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    command = reqbuf.command; 
	    len = reqbuf.length;
	
        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
	    data = &kernel_databuf[0];
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");

 	    // Write the data to the SmBus
        status = I2CWriteData(addr, command, data, len);      
        break;
      
	
       case I2C_SEND:      
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND IOCTL.\n");
       
        // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    len = reqbuf.length;
	
        // Get the user data to send (buffered I/O)
        if (copy_from_user (kernel_databuf, reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
	    data = &kernel_databuf[0];
	
	    command = reqbuf.command;     /* FxFx: Not Used but keep for debug signature */
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");
	
 	    // Send the data to the SmBus
        status = I2CSend(addr, data, len);      
        break;
       
     
       case I2C_SEND_COMMAND_THEN_READ_BLOCK:      
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND_COMMAND_THEN_READ_BLOCK IOCTL.\n");
       
        // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    command = reqbuf.command; 
	    len = reqbuf.length;
	
        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
	    data = &kernel_databuf[0]; 
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);
	
 	    // Read the data from the SmBus
        status = I2CReadDataBlock(addr, command, data, (int *)&len); 
        DumpBuffer((BYTE *)data, len, (BYTE *)"return..");
	
        // Return the data to the user (buffered I/O)
        if (copy_to_user (reqbuf.data, data, len)) {ret = -EFAULT; goto Exit_ioctl;}
			
        // Return the updated request data (ie length) to the user (buffered I/O)
        reqbuf.length = len;
        if (copy_to_user ((Req_Buf *)arg, &reqbuf, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
			
        break;       
    
    
       case I2C_SEND_COMMAND_THEN_WRITE_BLOCK:
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND_COMMAND_THEN_WRITE_BLOCK IOCTL.\n");
       
        // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    command = reqbuf.command; 
	    len = reqbuf.length;
	
        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
	    data = &kernel_databuf[0]; 
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");

 	    // Write the data to the SmBus
        status = I2CWriteDataBlock(addr, command, data, len);      
        break;
       
	
       case H8_Write:      
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  H8_Write IOCTL.\n");
        // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    len = reqbuf.length;
	
        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
	    data = &kernel_databuf[0]; 
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, len = %x, &data = %x\n", addr, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");

 	    // Write the data to the SmBus
        status = I2CH8Write(addr, data, len);
        DELAY(20000);
        break;
      
	
       case H8_ReadWrite:      
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  H8_ReadWrite IOCTL.\n");
         // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    len = reqbuf.length;
	
        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
	    data = &kernel_databuf[0]; 
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, len = %x, &data = %x\n", addr, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");
  		
 	    // Write the data to the SmBus
        status = I2CH8ReadWrite(addr, data, (int *)&len);
        debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  H8_ReadWrite Status = %x\n", status);
		// Dump the retruned data to log (Add one to the length parm so we log the checksum byte)
        DumpBuffer((BYTE *)data, len+1, (BYTE *)"return..");
		
        if (status == ERR_NO_ERROR)             //d36489 
        {
           if ((len > 0) && (len < 0xff))                    //d38299 and d64096		
           {
              // Add one so we return the checksum byte
              len++;
              // Return the data to the user (buffered I/O)
              if (copy_to_user (reqbuf.data, data, len)) {ret = -EFAULT; goto Exit_ioctl;}  
				
              // Return the updated request data (ie length) to the user (buffered I/O)
              reqbuf.length = len;
              if (copy_to_user ((Req_Buf *)arg, &reqbuf, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
           }
		   else
		   {
			   status = ERR_BUS_ERROR;
	           debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  H8_ReadWrite len = %x\n", len);
		   }			   
	    }   
        DELAY(20000);
   
        break;
       
       
       case GPIO_CHASSIS_CLEAR:      
	    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  GPIO_CHASSIS_CLEAR IOCTL.\n");
        break;
       
      default:  /* redundant, as cmd was checked against MAXNR */
        debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  Switch default.\n");
        ret = -ENOTTY; 
	    goto Exit_ioctl;	   
    }

    ret = status;
	
Exit_ioctl:	
    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  Exit RC=%x.\n\n", ret);

	// Release the driver lock
    up (&dev->sem);
    return ret;

}

#ifdef __x86_64__
#if defined(HAVE_COMPAT_IOCTL)
long
ibmsmb_ioctl_compat(struct file *filp,
		    unsigned int cmd,
		    unsigned long arg)
#else
int
handler_ioctl_32(unsigned int fd,
		 unsigned int cmd,
		 unsigned long arg,
		 struct file *filp)
#endif
{
    IbmSmb_Dev *dev = filp->private_data;
    int err= 0, ret = 0;

    Req_Buf_32 reqbuf;
    unsigned char kernel_databuf[KERNEL_DATABUF_SIZE];

    unsigned char addr = 0;
    unsigned char command = 0;
    unsigned long len = 0;         /* int ??? FxFx  */
    void *data = 0;
    I2CSTATUS status = 0;

    // Lock the driver via a Mutex to force serialize access by different processes
    if (down_interruptible (&dev->sem)) {
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  Signal received while blocking.\n");
                    return -ERESTARTSYS;
        }
    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  Enter.\n");

        /* accept only root */
    if (! capable(CAP_SYS_ADMIN))  {ret = -EACCES; goto Exit_ioctl;}

    /* don't even decode wrong cmds: better returning  ENOTTY than EFAULT */
    if (_IOC_TYPE(cmd) != IBMSMB_IOC_MAGIC) {ret = -ENOTTY; goto Exit_ioctl;}

    if (_IOC_NR(cmd) > IBMSMB_IOC_MAXNR) {ret = -ENOTTY; goto Exit_ioctl;}

    /*
     * the type is a bitmask, and VERIFY_WRITE catches R/W
     * transfers. Note that the type is user-oriented, while
     * verify_area is kernel-oriented, so the concept of "read" and
     * "write" is reversed
     */
    if (_IOC_DIR(cmd) & _IOC_READ)
        err = !access_ok(VERIFY_WRITE, (void *)arg, _IOC_SIZE(cmd));
    else if (_IOC_DIR(cmd) & _IOC_WRITE)
        err =  !access_ok(VERIFY_READ, (void *)arg, _IOC_SIZE(cmd));
    if (err) {ret = -EFAULT; goto Exit_ioctl;}
    
    // printk("********* PASSED ALL INITIAL 32 BIT HANDLER CHECKS ***********\n");

    // FxFx: Need to add paramater validation

    memset(&reqbuf, 0x00, sizeof(Req_Buf_32));
    memset(&kernel_databuf[0], 0x00, KERNEL_DATABUF_SIZE);

    switch(cmd) {

      //  USING THIS AS THE WAY TO SET DEBUG TRACE LEVEL
      case IBMSMB_IOCTEST1_32:
        if (copy_from_user (&reqbuf, (Req_Buf_32 *)arg, sizeof(Req_Buf_32))) {ret = -EFAULT; goto Exit_ioctl;}
        // Reuse the addr field of the reqbuf as the way to specify the debug level
        printk("Debug_Level before = %d\n", Debug_Level);
        Debug_Level = reqbuf.addr;
        printk("Debug_Level after = %d\n", Debug_Level);

        break;
                                                                                                                                                             
      //  USING THIS AS THE KLOG TEST POINT (for user space synchronized logging) 
      case IBMSMB_IOCTEST2_32:
        // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            len = reqbuf.length;

        // Get the user data to syslog (buffered I/O)
        if (copy_from_user (kernel_databuf, (unsigned char *)(unsigned long)reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
            data = &kernel_databuf[0];

            // Write the data to the syslog if debug messages are enabled
        if (Debug_Level >= INTERFACE_LVL)
                printk("%s\n", (unsigned char *)data);

        break;
                                                                                                                                                             
      case I2C_RECEIVE_32:
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_RECEIVE IOCTL.\n");

        // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            len = reqbuf.length;
            data = &kernel_databuf[0];

            command = reqbuf.command;   // FxFx: Not Used but keep for debug signature 
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);

            // Receive the data from the SmBus
        status = I2CReceive(addr, data, len);
        DumpBuffer((BYTE *)data, len, (BYTE *)"return..");

        // Return the data to the user (buffered I/O)
        if (copy_to_user ((unsigned char *)(unsigned long)reqbuf.data, data, len)) {ret = -EFAULT; goto Exit_ioctl;}
        break;
                                                                                                                                                             
      case I2C_SEND_COMMAND_THEN_READ_32:
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND_COMMAND_THEN_READ IOCTL.\n");

        // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf_32 *)arg, sizeof(Req_Buf_32))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            command = reqbuf.command;
            len = (unsigned long)reqbuf.length;
            data = &kernel_databuf[0];
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);

            // Read the data from the SmBus
        status = I2CReadData(addr, command, data, len);
        DumpBuffer((BYTE *)data, len, (BYTE *)"return..");

        // Return the data to the user (buffered I/O)
        if (copy_to_user ((unsigned char *)(unsigned long)reqbuf.data, data, len)) {ret = -EFAULT; goto Exit_ioctl;}
        break;


      case I2C_SEND_COMMAND_THEN_WRITE_32:
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND_COMMAND_THEN_WRITE IOCTL.\n");

        // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            command = reqbuf.command;
            len = reqbuf.length;

        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, (unsigned char *)(unsigned long)reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
            data = &kernel_databuf[0];
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");

            // Write the data to the SmBus
        status = I2CWriteData(addr, command, data, len);
        break;


       case I2C_SEND_32:
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND IOCTL.\n");

        // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            len = reqbuf.length;

        // Get the user data to send (buffered I/O)
        if (copy_from_user (kernel_databuf, (unsigned char *)(unsigned long)reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
            data = &kernel_databuf[0];

            command = reqbuf.command;     // FxFx: Not Used but keep for debug signature 
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");

            // Send the data to the SmBus
        status = I2CSend(addr, data, len);
        break;
                                                                                                                                                             
       case I2C_SEND_COMMAND_THEN_READ_BLOCK_32:
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND_COMMAND_THEN_READ_BLOCK IOCTL.\n");

        // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            command = reqbuf.command;
            len = reqbuf.length;

        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, (unsigned char *)(unsigned long)reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
            data = &kernel_databuf[0];
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);

            // Read the data from the SmBus
        status = I2CReadDataBlock(addr, command, data, (int *)&len);
        DumpBuffer((BYTE *)data, len, (BYTE *)"return..");

        // Return the data to the user (buffered I/O)
        if (copy_to_user ((unsigned char *)(unsigned long)reqbuf.data, data, len)) {ret = -EFAULT; goto Exit_ioctl;}

        // Return the updated request data (ie length) to the user (buffered I/O)
        reqbuf.length = len;
        if (copy_to_user ((Req_Buf *)arg, &reqbuf, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}

        break;
                                                                                                                                                             
       case I2C_SEND_COMMAND_THEN_WRITE_BLOCK_32:
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  I2C_SEND_COMMAND_THEN_WRITE_BLOCK IOCTL.\n");

        // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            command = reqbuf.command;
            len = reqbuf.length;

        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, (unsigned char *)(unsigned long)reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
            data = &kernel_databuf[0];
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, command = %x, len = %x, &data = %x\n", addr, command, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");

            // Write the data to the SmBus
        status = I2CWriteDataBlock(addr, command, data, len);
        break;
                                                                                                                                                             
       case H8_Write_32:
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  H8_Write IOCTL.\n");
        // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            len = reqbuf.length;

        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, (unsigned char *)(unsigned long)reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
            data = &kernel_databuf[0];
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, len = %x, &data = %x\n", addr, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");

            // Write the data to the SmBus
        status = I2CH8Write(addr, data, len);
        DELAY(20000);
        break;


       case H8_ReadWrite_32:
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  H8_ReadWrite IOCTL.\n");
         // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf_32 *)arg, sizeof(Req_Buf_32))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            len = reqbuf.length;

        // Get the user data to write (buffered I/O)
        if (copy_from_user (kernel_databuf, (unsigned char *)(unsigned long)reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
            data = &kernel_databuf[0];
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  addr = %x, len = %x, &data = %x\n", addr, (int)len, data);
        DumpBuffer((BYTE *)data, len, (BYTE *)"passed..");

            // Write the data to the SmBus
        status = I2CH8ReadWrite(addr, data, (int *)&len);
        debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  H8_ReadWrite Status = %x\n", status);
                // Dump the retruned data to log (Add one to the length parm so we log the checksum byte)
        DumpBuffer((BYTE *)data, len+1, (BYTE *)"return..");

        if (status == ERR_NO_ERROR)             //d36489
        {
           if ((len > 0) && (len < 0xff))                    //d38299 and d64096		
           {
              // Add one so we return the checksum byte
              len++;
              // Return the data to the user (buffered I/O)
              if (copy_to_user ((unsigned char *)(unsigned long)reqbuf.data, data, len)) {ret = -EFAULT; goto Exit_ioctl;}

              // Return the updated request data (ie length) to the user (buffered I/O)
              reqbuf.length = len;
              if (copy_to_user ((Req_Buf *)arg, &reqbuf, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
           }
                   else
                   {
                           status = ERR_BUS_ERROR;
                   debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  H8_ReadWrite len = %x\n", len);
                   }
            }
        DELAY(20000);

        break;
       

       case GPIO_CHASSIS_CLEAR_32:
            debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  GPIO_CHASSIS_CLEAR IOCTL.\n");
        break;

      default:  // redundant, as cmd was checked against MAXNR 
        debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  Switch default.\n");
        ret = -ENOTTY;
            goto Exit_ioctl;
    }
                                                                                                                                                             
    ret = status;


Exit_ioctl:
    debug_out(INTERFACE_LVL,"ibmsmb_ioctl.  Exit RC=%x.\n\n", ret);
                                                                                                                                                             
        // Release the driver lock
    up (&dev->sem);
    return ret;

}
#endif


// File operations
//---------------------------------------------------------------------------------------
struct file_operations ibmsmb_fops = {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
    owner : THIS_MODULE,
#endif
#if defined(HAVE_UNLOCKED_IOCTL)
    unlocked_ioctl: ibmsmb_ioctl_unlocked,
#else
    ioctl: ibmsmb_ioctl,
#endif
#ifdef __x86_64__
#if defined(HAVE_COMPAT_IOCTL)
    compat_ioctl: ibmsmb_ioctl_compat,
#endif
#endif
    open:  ibmsmb_open,
    release: ibmsmb_release,
};



// Utility functions 
//---------------------------------------------------------------------------------------
int ibmsmb_trim(IbmSmb_Dev *dev)
{
    IbmSmb_Dev *next, *dptr;
    int qset = dev->qset;   /* "dev" is not-null */
    int i;

    debug_out(INTERFACE_LVL,"ibmsmb_trim.  Enter.\n");
	
    if (dev->vmas) /* don't trim: there are active mappings */
        return -EBUSY;

    for (dptr = dev; dptr; dptr = next) { /* all the list items */
        if (dptr->data) {
            for (i = 0; i < qset; i++)
                if (dptr->data[i])
                    kmem_cache_free(ibmsmb_cache, dptr->data[i]);

            kfree(dptr->data);
            dptr->data=NULL;
        }
        next=dptr->next;
        if (dptr != dev) kfree(dptr); /* all of them but the first */
    }
    dev->size = 0;
    dev->qset = ibmsmb_qset;
    dev->quantum = ibmsmb_quantum;
    dev->next = NULL;
    return 0;
}




/*
 * Module load and Unload
 */
//---------------------------------------------------------------------------------------
int ibmsmb_init(void)
{
    int result, i;

    //printk("IBMSMB:ibmsmb_init.  Enter.\n");

    SET_MODULE_OWNER(&ibmsmb_fops);
    /*
     * Register your major, and accept a dynamic number
     */
    result = register_chrdev(ibmsmb_major, "ibmsmb", &ibmsmb_fops);
    if (result < 0) return result;
    if (ibmsmb_major == 0) ibmsmb_major = result; /* dynamic */
		
    //printk("IBMSMB:ibmsmb_init.  ibmsmb_major = %d\n", ibmsmb_major);
	
    /* 
     * allocate the devices -- we can't have them static, as the number
     * can be specified at load time
     */
    ibmsmb_devices = kmalloc(ibmsmb_devs * sizeof (IbmSmb_Dev), GFP_KERNEL);
    if (!ibmsmb_devices) {
        result = -ENOMEM;
        goto fail_malloc;
    }
    memset(ibmsmb_devices, 0, ibmsmb_devs * sizeof (IbmSmb_Dev));
    for (i=0; i < ibmsmb_devs; i++) {
        ibmsmb_devices[i].quantum = ibmsmb_quantum;
        ibmsmb_devices[i].qset = ibmsmb_qset;
        sema_init (&ibmsmb_devices[i].sem, 1);
    }

    /* init_module: create a cache for our quanta */
    ibmsmb_cache =
	kmem_cache_create("ibmsmb", ibmsmb_quantum,
			  0, SLAB_HWCACHE_ALIGN,
			  NULL, NULL); /* no ctor/dtor */
    if (!ibmsmb_cache) {
        result = -ENOMEM;
        goto fail_malloc2;
    }

	// Initialize the hardware
    I2CInit(0);
	
    // Set up buffer dumping to the syslog
    InitxHexxPairxs(); 

#ifdef __x86_64__
#if !defined(HAVE_COMPAT_IOCTL)
        /*
         * Register the 32-bit ioctl conversion if on 64bit systems
         */
        register_ioctl32_conversion( IBMSMB_IOCTEST1_32, handler_ioctl_32 );
        register_ioctl32_conversion( I2C_RECEIVE_32, handler_ioctl_32 );
        register_ioctl32_conversion( I2C_SEND_COMMAND_THEN_READ_32, handler_ioctl_32 );
        register_ioctl32_conversion( I2C_SEND_COMMAND_THEN_WRITE_32, handler_ioctl_32 );
        register_ioctl32_conversion( I2C_SEND_32, handler_ioctl_32 );
        register_ioctl32_conversion( I2C_SEND_COMMAND_THEN_READ_BLOCK_32, handler_ioctl_32 );
        register_ioctl32_conversion( I2C_SEND_COMMAND_THEN_WRITE_BLOCK_32, handler_ioctl_32 );
        register_ioctl32_conversion( H8_Write_32, handler_ioctl_32 );
        register_ioctl32_conversion( H8_ReadWrite_32, handler_ioctl_32 );
#endif
#endif
	
    //printk("IBMSMB:ibmsmb_init.  Exit - Good.\n");
    return 0; /* succeed */

  fail_malloc2:
    kfree(ibmsmb_devices);
  fail_malloc:
    unregister_chrdev(ibmsmb_major, "ibmsmb");

    //printk("IBMSMB:ibmsmb_init.  Exit - Fail.\n");
    return result;
}


//---------------------------------------------------------------------------------------
void ibmsmb_cleanup(void)
{
    int i;

    debug_out(INTERFACE_LVL, "ibmsmb_cleanup.  Enter.\n");

#ifdef __x86_64__
#if !defined(HAVE_COMPAT_IOCTL)
    /*
     * Un-Register the 32-bit ioctl conversion if on 64bit systems
     */
    unregister_ioctl32_conversion( IBMSMB_IOCTEST1_32 );
    unregister_ioctl32_conversion( I2C_RECEIVE_32 );
    unregister_ioctl32_conversion( I2C_SEND_COMMAND_THEN_READ_32 );
    unregister_ioctl32_conversion( I2C_SEND_COMMAND_THEN_WRITE_32  );
    unregister_ioctl32_conversion( I2C_SEND_32 );
    unregister_ioctl32_conversion( I2C_SEND_COMMAND_THEN_READ_BLOCK_32 );
    unregister_ioctl32_conversion( I2C_SEND_COMMAND_THEN_WRITE_BLOCK_32 );
    unregister_ioctl32_conversion( H8_Write_32 );
    unregister_ioctl32_conversion( H8_ReadWrite_32 );
#endif
#endif


    unregister_chrdev(ibmsmb_major, "ibmsmb");

    for (i=0; i<ibmsmb_devs; i++)
        ibmsmb_trim(ibmsmb_devices+i);
    kfree(ibmsmb_devices);

    /* cleanup_module: release the cache of our quanta */
    kmem_cache_destroy(ibmsmb_cache);

    debug_out(INTERFACE_LVL,"ibmsmb_cleanup.  Exit.\n");
}

#endif /* not linux-2.0 */


//---------------------------------------------------------------------------------------
module_init(ibmsmb_init);
module_exit(ibmsmb_cleanup);
